# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Hosts


class HostsTestCase(unittest.TestCase):
    def test_create_host(self):
        request = Hosts.create_host("host_name")
        self.assertEqual(
            bytes(request),
            b"<create_asset><asset>"
            b"<type>host</type>"
            b"<name>host_name</name>"
            b"</asset></create_asset>",
        )

    def test_create_host_with_comment(self):
        request = Hosts.create_host("host_name", comment="comment")
        self.assertEqual(
            bytes(request),
            b"<create_asset><asset>"
            b"<type>host</type>"
            b"<name>host_name</name>"
            b"<comment>comment</comment>"
            b"</asset></create_asset>",
        )

    def test_create_host_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Hosts.create_host(None)

        with self.assertRaises(RequiredArgument):
            Hosts.create_host("")

    def test_delete_host(self):
        request = Hosts.delete_host("host_id")
        self.assertEqual(
            bytes(request),
            b'<delete_asset asset_id="host_id"/>',
        )

    def test_delete_host_missing_host_id(self):
        with self.assertRaises(RequiredArgument):
            Hosts.delete_host(None)

        with self.assertRaises(RequiredArgument):
            Hosts.delete_host("")

    def test_get_hosts(self):
        request = Hosts.get_hosts()
        self.assertEqual(
            bytes(request),
            b'<get_assets type="host"/>',
        )

    def test_get_hosts_with_filter_string(self):
        request = Hosts.get_hosts(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_assets type="host" filter="filter_string"/>',
        )

    def test_get_hosts_with_filter_id(self):
        request = Hosts.get_hosts(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_assets type="host" filt_id="filter_id"/>',
        )

    def test_get_hosts_with_details(self):
        request = Hosts.get_hosts(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_assets type="host" details="1"/>',
        )

        request = Hosts.get_hosts(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_assets type="host" details="0"/>',
        )

    def test_get_host(self):
        request = Hosts.get_host("host_id")
        self.assertEqual(
            bytes(request),
            b'<get_assets asset_id="host_id" type="host"/>',
        )

    def test_get_host_with_details(self):
        request = Hosts.get_host("host_id", details=True)
        self.assertEqual(
            bytes(request),
            b'<get_assets asset_id="host_id" type="host" details="1"/>',
        )

        request = Hosts.get_host("host_id", details=False)
        self.assertEqual(
            bytes(request),
            b'<get_assets asset_id="host_id" type="host" details="0"/>',
        )

    def test_get_host_missing_host_id(self):
        with self.assertRaises(RequiredArgument):
            Hosts.get_host(None)

        with self.assertRaises(RequiredArgument):
            Hosts.get_host("")

    def test_modify_host(self):
        request = Hosts.modify_host("host_id")
        self.assertEqual(
            bytes(request),
            b'<modify_asset asset_id="host_id">'
            b"<comment></comment>"
            b"</modify_asset>",
        )

    def test_modify_host_with_comment(self):
        request = Hosts.modify_host("host_id", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_asset asset_id="host_id">'
            b"<comment>comment</comment>"
            b"</modify_asset>",
        )

    def test_modify_host_missing_host_id(self):
        with self.assertRaises(RequiredArgument):
            Hosts.modify_host(None)

        with self.assertRaises(RequiredArgument):
            Hosts.modify_host("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Cves


class CvesTestCase(unittest.TestCase):
    def test_get_cves(self):
        request = Cves.get_cves()
        self.assertEqual(
            bytes(request),
            b'<get_info type="CVE"/>',
        )

    def test_get_cves_with_filter_string(self):
        request = Cves.get_cves(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_info type="CVE" filter="filter_string"/>',
        )

    def test_get_cves_with_filter_id(self):
        request = Cves.get_cves(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_info type="CVE" filt_id="filter_id"/>',
        )

    def test_get_cves_with_name(self):
        request = Cves.get_cves(name="name")
        self.assertEqual(
            bytes(request),
            b'<get_info type="CVE" name="name"/>',
        )

    def test_get_cves_with_details(self):
        request = Cves.get_cves(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_info type="CVE" details="1"/>',
        )

        request = Cves.get_cves(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_info type="CVE" details="0"/>',
        )

    def test_get_cve(self):
        request = Cves.get_cve("cve_id")
        self.assertEqual(
            bytes(request),
            b'<get_info info_id="cve_id" type="CVE" details="1"/>',
        )

    def test_get_cve_missing_cve_id(self):
        with self.assertRaises(RequiredArgument):
            Cves.get_cve(None)

        with self.assertRaises(RequiredArgument):
            Cves.get_cve("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Cpes


class CpesTestCase(unittest.TestCase):
    def test_get_cpes(self):
        request = Cpes.get_cpes()
        self.assertEqual(
            bytes(request),
            b'<get_info type="CPE"/>',
        )

    def test_get_cpes_with_filter_string(self):
        request = Cpes.get_cpes(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_info type="CPE" filter="filter_string"/>',
        )

    def test_get_cpes_with_filter_id(self):
        request = Cpes.get_cpes(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_info type="CPE" filt_id="filter_id"/>',
        )

    def test_get_cpes_with_name(self):
        request = Cpes.get_cpes(name="name")
        self.assertEqual(
            bytes(request),
            b'<get_info type="CPE" name="name"/>',
        )

    def test_get_cpes_with_details(self):
        request = Cpes.get_cpes(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_info type="CPE" details="1"/>',
        )

        request = Cpes.get_cpes(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_info type="CPE" details="0"/>',
        )

    def test_get_cpe(self):
        request = Cpes.get_cpe("cpe_id")
        self.assertEqual(
            bytes(request),
            b'<get_info info_id="cpe_id" type="CPE" details="1"/>',
        )

    def test_get_cpe_missing_cpe_id(self):
        with self.assertRaises(RequiredArgument):
            Cpes.get_cpe(None)

        with self.assertRaises(RequiredArgument):
            Cpes.get_cpe("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.operating_systems import (
    GmpDeleteOperatingSystemTestMixin,
    GmpGetOperatingSystemsTestMixin,
    GmpGetOperatingSystemTestMixin,
    GmpModifyOperatingSystemTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPDeleteOperatingSystemTestCase(
    GmpDeleteOperatingSystemTestMixin, GMPTestCase
):
    pass


class GMPGetOperatingSystemTestCase(
    GmpGetOperatingSystemTestMixin, GMPTestCase
):
    pass


class GMPGetOperatingSystemsTestCase(
    GmpGetOperatingSystemsTestMixin, GMPTestCase
):
    pass


class GMPModifyOperatingSystemTestCase(
    GmpModifyOperatingSystemTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tickets import (
    GmpCloneTicketTestMixin,
    GmpCreateTicketTestMixin,
    GmpDeleteTicketTestMixin,
    GmpGetTicketsTestMixin,
    GmpGetTicketTestMixin,
    GmpModifyTicketTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPDeleteTicketTestCase(GmpDeleteTicketTestMixin, GMPTestCase):
    pass


class GMPGetTicketTestCase(GmpGetTicketTestMixin, GMPTestCase):
    pass


class GMPGetTicketsTestCase(GmpGetTicketsTestMixin, GMPTestCase):
    pass


class GMPCloneTicketTestCase(GmpCloneTicketTestMixin, GMPTestCase):
    pass


class GMPCreateTicketTestCase(GmpCreateTicketTestMixin, GMPTestCase):
    pass


class GMPModifyTicketTestCase(GmpModifyTicketTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_ticket import GmpCloneTicketTestMixin
from .test_create_ticket import GmpCreateTicketTestMixin
from .test_delete_ticket import GmpDeleteTicketTestMixin
from .test_get_ticket import GmpGetTicketTestMixin
from .test_get_tickets import GmpGetTicketsTestMixin
from .test_modify_ticket import GmpModifyTicketTestMixin

__all__ = (
    "GmpCloneTicketTestMixin",
    "GmpCreateTicketTestMixin",
    "GmpDeleteTicketTestMixin",
    "GmpGetTicketTestMixin",
    "GmpGetTicketsTestMixin",
    "GmpModifyTicketTestMixin",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from decimal import Decimal

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import Notes


class NotesTestCase(unittest.TestCase):
    def test_create_note(self):
        request = Notes.create_note("foo", nvt_oid="oid1")

        self.assertEqual(
            bytes(request),
            b'<create_note><text>foo</text><nvt oid="oid1"/></create_note>',
        )

    def test_create_note_missing_text(self):
        with self.assertRaises(RequiredArgument):
            Notes.create_note(None, "od1")

        with self.assertRaises(RequiredArgument):
            Notes.create_note("", "oid1")

    def test_create_note_missing_nvt_oid(self):
        with self.assertRaises(RequiredArgument):
            Notes.create_note("foo", None)

        with self.assertRaises(RequiredArgument):
            Notes.create_note("foo", "")

    def test_create_note_with_hosts(self):
        request = Notes.create_note("foo", nvt_oid="oid1", hosts=[])

        self.assertEqual(
            bytes(request),
            b'<create_note><text>foo</text><nvt oid="oid1"/></create_note>',
        )

        request = Notes.create_note("foo", nvt_oid="oid1", hosts=["h1", "h2"])

        self.assertEqual(
            bytes(request),
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<hosts>h1,h2</hosts>"
            b"</create_note>",
        )

    def test_create_note_with_port(self):
        request = Notes.create_note("foo", nvt_oid="oid1", port="666/tcp")

        self.assertEqual(
            bytes(request),
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<port>666/tcp</port>"
            b"</create_note>",
        )

    def test_create_note_with_result_id(self):
        request = Notes.create_note("foo", nvt_oid="oid1", result_id="r1")

        self.assertEqual(
            bytes(request),
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b'<result id="r1"/>'
            b"</create_note>",
        )

    def test_create_note_with_task_id(self):
        request = Notes.create_note("foo", nvt_oid="oid1", task_id="t1")

        self.assertEqual(
            bytes(request),
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b'<task id="t1"/>'
            b"</create_note>",
        )

    def test_create_note_with_severity(self):
        request = Notes.create_note("foo", nvt_oid="oid1", severity="5.5")

        self.assertEqual(
            bytes(request),
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<severity>5.5</severity>"
            b"</create_note>",
        )

    def test_create_note_with_days_active(self):
        request = Notes.create_note("foo", nvt_oid="oid1", days_active=5)

        self.assertEqual(
            bytes(request),
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<active>5</active>"
            b"</create_note>",
        )

    def test_create_note_with_invalid_port(self):
        with self.assertRaises(InvalidArgument):
            Notes.create_note("foo", nvt_oid="oid1", port="666")

        with self.assertRaises(InvalidArgument):
            Notes.create_note("foo", nvt_oid="oid1", port="666/")

        with self.assertRaises(InvalidArgument):
            Notes.create_note("foo", nvt_oid="oid1", port="udp/666")

    def test_modify_note(self):
        request = Notes.modify_note("n1", "foo")

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1"><text>foo</text></modify_note>',
        )

    def test_modify_note_missing_note_id(self):
        with self.assertRaises(RequiredArgument):
            Notes.modify_note(None, "foo")

        with self.assertRaises(RequiredArgument):
            Notes.modify_note("", "foo")

    def test_modify_note_missing_text(self):
        with self.assertRaises(RequiredArgument):
            Notes.modify_note("n1", None)

        with self.assertRaises(RequiredArgument):
            Notes.modify_note("n1", "")

    def test_modify_note_with_days_active(self):
        request = Notes.modify_note("n1", "foo", days_active=0)

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<active>0</active>"
            b"</modify_note>",
        )

        request = Notes.modify_note("n1", "foo", days_active=-1)

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<active>-1</active>"
            b"</modify_note>",
        )

        request = Notes.modify_note("n1", "foo", days_active=600)

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<active>600</active>"
            b"</modify_note>",
        )

    def test_modify_note_with_port(self):
        request = Notes.modify_note("n1", "foo", port="123/tcp")

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<port>123/tcp</port>"
            b"</modify_note>",
        )

    def test_modify_note_with_hosts(self):
        request = Notes.modify_note("n1", "foo", hosts=["h1"])

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<hosts>h1</hosts>"
            b"</modify_note>",
        )

        request = Notes.modify_note("n1", "foo", hosts=["h1", "h2"])

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<hosts>h1,h2</hosts>"
            b"</modify_note>",
        )

    def test_modify_note_with_result_id(self):
        request = Notes.modify_note("n1", "foo", result_id="r1")

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b'<result id="r1"/>'
            b"</modify_note>",
        )

    def test_modify_note_with_task_id(self):
        request = Notes.modify_note("n1", "foo", task_id="t1")

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b'<task id="t1"/>'
            b"</modify_note>",
        )

    def test_modify_note_with_severity(self):
        request = Notes.modify_note("n1", "foo", severity="5.5")

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_note>",
        )

        request = Notes.modify_note("n1", "foo", severity=5.5)

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_note>",
        )

        request = Notes.modify_note("n1", "foo", severity=Decimal("5.5"))

        self.assertEqual(
            bytes(request),
            b'<modify_note note_id="n1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_note>",
        )

    def test_modify_note_with_invalid_port(self):
        with self.assertRaises(InvalidArgument):
            Notes.modify_note("n1", "foo", port="666")

        with self.assertRaises(InvalidArgument):
            Notes.modify_note("n1", "foo", port="666/")

        with self.assertRaises(InvalidArgument):
            Notes.modify_note("n1", "foo", port="udp/666")

    def test_clone_note(self):
        request = Notes.clone_note("n1")

        self.assertEqual(
            bytes(request),
            b"<create_note><copy>n1</copy></create_note>",
        )

    def test_clone_note_missing_note_id(self):
        with self.assertRaises(RequiredArgument):
            Notes.clone_note(None)

        with self.assertRaises(RequiredArgument):
            Notes.clone_note("")

    def test_delete_note(self):
        request = Notes.delete_note("n1")

        self.assertEqual(
            bytes(request), b'<delete_note note_id="n1" ultimate="0"/>'
        )

    def test_delete_note_ultimate(self):
        request = Notes.delete_note("n1", ultimate=True)

        self.assertEqual(
            bytes(request), b'<delete_note note_id="n1" ultimate="1"/>'
        )

        request = Notes.delete_note("n1", ultimate=False)

        self.assertEqual(
            bytes(request), b'<delete_note note_id="n1" ultimate="0"/>'
        )

    def test_delete_note_missing_note_id(self):
        with self.assertRaises(RequiredArgument):
            Notes.delete_note(None)

        with self.assertRaises(RequiredArgument):
            Notes.delete_note("")

    def test_get_notes(self):
        request = Notes.get_notes()

        self.assertEqual(bytes(request), b"<get_notes/>")

    def test_get_notes_with_filter_string(self):
        request = Notes.get_notes(filter_string="foo=bar")

        self.assertEqual(bytes(request), b'<get_notes filter="foo=bar"/>')

    def test_get_notes_with_filter_id(self):
        request = Notes.get_notes(filter_id="f1")

        self.assertEqual(bytes(request), b'<get_notes filt_id="f1"/>')

    def test_get_notes_with_details(self):
        request = Notes.get_notes(details=True)

        self.assertEqual(bytes(request), b'<get_notes details="1"/>')

        request = Notes.get_notes(details=False)

        self.assertEqual(bytes(request), b'<get_notes details="0"/>')

    def test_get_notes_with_result(self):
        request = Notes.get_notes(result=True)

        self.assertEqual(bytes(request), b'<get_notes result="1"/>')

        request = Notes.get_notes(result=False)

        self.assertEqual(bytes(request), b'<get_notes result="0"/>')

    def test_get_note(self):
        request = Notes.get_note("n1")

        self.assertEqual(
            bytes(request), b'<get_notes note_id="n1" details="1"/>'
        )

    def test_get_note_missing_note_id(self):
        with self.assertRaises(RequiredArgument):
            Notes.get_note(None)

        with self.assertRaises(RequiredArgument):
            Notes.get_note("")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpDeleteScannerTestMixin:
    def test_delete(self):
        self.gmp.delete_scanner("a1")

        self.connection.send.has_been_called_with(
            b'<delete_scanner scanner_id="a1" ultimate="0"/>'
        )

    def test_delete_ultimate(self):
        self.gmp.delete_scanner("a1", ultimate=True)

        self.connection.send.has_been_called_with(
            b'<delete_scanner scanner_id="a1" ultimate="1"/>'
        )

    def test_missing_id(self):
        with self.assertRaises(GvmError):
            self.gmp.delete_scanner(None)

        with self.assertRaises(GvmError):
            self.gmp.delete_scanner("")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


class GmpGetAgentGroupsTestMixin:
    def test_get_agent_groups(self):
        self.gmp.get_agent_groups()

        self.connection.send.has_been_called_with(b"<get_agent_groups/>")

    def test_get_agent_groups_with_filter_string(self):
        self.gmp.get_agent_groups(filter_string="name=group")

        self.connection.send.has_been_called_with(
            b'<get_agent_groups filter="name=group"/>'
        )

    def test_get_agent_groups_with_filter_id(self):
        self.gmp.get_agent_groups(filter_id="filter-1")

        self.connection.send.has_been_called_with(
            b'<get_agent_groups filt_id="filter-1"/>'
        )

    def test_get_agent_groups_with_trash(self):
        self.gmp.get_agent_groups(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_agent_groups trash="1"/>'
        )

    def test_get_agent_groups_with_trash_false(self):
        self.gmp.get_agent_groups(trash=False)

        self.connection.send.has_been_called_with(
            b'<get_agent_groups trash="0"/>'
        )

    def test_get_agent_groups_without_trash(self):
        self.gmp.get_agent_groups()

        self.connection.send.has_been_called_with(b"<get_agent_groups/>")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import OperatingSystems


class OperatingSystemsTestCase(unittest.TestCase):
    def test_delete_operating_system(self):
        request = OperatingSystems.delete_operating_system(
            "operating_system_id"
        )
        self.assertEqual(
            bytes(request),
            b'<delete_asset asset_id="operating_system_id"/>',
        )

    def test_delete_operating_system_missing_operating_system_id(self):
        with self.assertRaises(RequiredArgument):
            OperatingSystems.delete_operating_system(None)

        with self.assertRaises(RequiredArgument):
            OperatingSystems.delete_operating_system("")

    def test_get_operating_systems(self):
        request = OperatingSystems.get_operating_systems()
        self.assertEqual(
            bytes(request),
            b'<get_assets type="os"/>',
        )

    def test_get_operating_systems_with_filter_string(self):
        request = OperatingSystems.get_operating_systems(
            filter_string="filter_string"
        )
        self.assertEqual(
            bytes(request),
            b'<get_assets type="os" filter="filter_string"/>',
        )

    def test_get_operating_systems_with_filter_id(self):
        request = OperatingSystems.get_operating_systems(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_assets type="os" filt_id="filter_id"/>',
        )

    def test_get_operating_systems_with_details(self):
        request = OperatingSystems.get_operating_systems(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_assets type="os" details="1"/>',
        )

        request = OperatingSystems.get_operating_systems(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_assets type="os" details="0"/>',
        )

    def test_get_operating_system(self):
        request = OperatingSystems.get_operating_system("operating_system_id")
        self.assertEqual(
            bytes(request),
            b'<get_assets asset_id="operating_system_id" type="os"/>',
        )

    def test_get_operating_system_with_details(self):
        request = OperatingSystems.get_operating_system(
            "operating_system_id", details=True
        )
        self.assertEqual(
            bytes(request),
            b'<get_assets asset_id="operating_system_id" type="os" details="1"/>',
        )
        request = OperatingSystems.get_operating_system(
            "operating_system_id", details=False
        )
        self.assertEqual(
            bytes(request),
            b'<get_assets asset_id="operating_system_id" type="os" details="0"/>',
        )

    def test_modify_operating_system(self):
        request = OperatingSystems.modify_operating_system(
            "operating_system_id"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_asset asset_id="operating_system_id">'
            b"<comment></comment>"
            b"</modify_asset>",
        )

    def test_modify_operating_system_with_comment(self):
        request = OperatingSystems.modify_operating_system(
            "operating_system_id", comment="comment"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_asset asset_id="operating_system_id">'
            b"<comment>comment</comment>"
            b"</modify_asset>",
        )

    def test_modify_operating_system_missing_operating_system_id(self):
        with self.assertRaises(RequiredArgument):
            OperatingSystems.modify_operating_system(None)

        with self.assertRaises(RequiredArgument):
            OperatingSystems.modify_operating_system("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tags import (
    GmpCloneTagTestMixin,
    GmpCreateTagTestMixin,
    GmpDeleteTagTestMixin,
    GmpGetTagsTestMixin,
    GmpGetTagTestMixin,
    GmpModifyTagTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPDeleteTagTestCase(GmpDeleteTagTestMixin, GMPTestCase):
    pass


class GMPGetTagTestCase(GmpGetTagTestMixin, GMPTestCase):
    pass


class GMPGetTagsTestCase(GmpGetTagsTestMixin, GMPTestCase):
    pass


class GMPCloneTagTestCase(GmpCloneTagTestMixin, GMPTestCase):
    pass


class GMPCreateTagTestCase(GmpCreateTagTestMixin, GMPTestCase):
    pass


class GMPModifyTagTestCase(GmpModifyTagTestMixin, GMPTestCase):
    pass



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyOCIImageTargetTestMixin:
    def test_modify_oci_image_target(self):
        self.gmp.modify_oci_image_target(oci_image_target_id="t1")

        self.connection.send.has_been_called_with(
            b'<modify_oci_image_target oci_image_target_id="t1"/>'
        )

    def test_modify_target_missing_target_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_oci_image_target(oci_image_target_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_oci_image_target(oci_image_target_id="")

    def test_modify_target_with_comment(self):
        self.gmp.modify_oci_image_target(
            oci_image_target_id="t1", comment="foo"
        )

        self.connection.send.has_been_called_with(
            b'<modify_oci_image_target oci_image_target_id="t1">'
            b"<comment>foo</comment>"
            b"</modify_oci_image_target>"
        )

    def test_modify_target_with_image_references(self):
        self.gmp.modify_oci_image_target(
            oci_image_target_id="t1", image_references=["oci://foo/bar:latest"]
        )

        self.connection.send.has_been_called_with(
            b'<modify_oci_image_target oci_image_target_id="t1">'
            b"<image_references>oci://foo/bar:latest</image_references>"
            b"</modify_oci_image_target>"
        )

        self.gmp.modify_oci_image_target(
            oci_image_target_id="t1",
            image_references=["oci://foo/bar:latest", "oci://baz/qux:latest"],
        )

        self.connection.send.has_been_called_with(
            b'<modify_oci_image_target oci_image_target_id="t1">'
            b"<image_references>oci://foo/bar:latest,oci://baz/qux:latest</image_references>"
            b"</modify_oci_image_target>"
        )

    def test_modify_target_with_name(self):
        self.gmp.modify_oci_image_target(oci_image_target_id="t1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_oci_image_target oci_image_target_id="t1">'
            b"<name>foo</name>"
            b"</modify_oci_image_target>"
        )

    def test_modify_target_with_smb_credential_id(self):
        self.gmp.modify_oci_image_target(
            oci_image_target_id="t1", credential_id="c1"
        )

        self.connection.send.has_been_called_with(
            b'<modify_oci_image_target oci_image_target_id="t1">'
            b'<credential id="c1"/>'
            b"</modify_oci_image_target>"
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v224 import HelpFormat


class GmpHelpTestMixin:
    def test_help(self):
        self.gmp.help()

        self.connection.send.has_been_called_with(b'<help type=""/>')

    def test_help_type_brief(self):
        self.gmp.help(brief=True)

        self.connection.send.has_been_called_with(b'<help type="brief"/>')

    def test_invalid_help_format(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.help(help_format="foo")

    def test_html_format(self):
        self.gmp.help(help_format=HelpFormat.HTML)

        self.connection.send.has_been_called_with(
            b'<help type="" format="html"/>'
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm._enum import Enum
from gvm.errors import InvalidArgument


class SomeEnum(Enum):
    FOO = "foo"
    BAR = "bar"
    LOREM = "ipsum"


class OtherEnum(Enum):
    LOREM = "ipsum"


class SomeClass:
    def __str__(self) -> str:
        return "foo"


class EnumTestCase(unittest.TestCase):
    def test_enum(self) -> None:
        enum = SomeEnum("FOO")
        self.assertEqual(enum, SomeEnum.FOO)
        enum = SomeEnum("BAR")
        self.assertEqual(enum, SomeEnum.BAR)

        enum = SomeEnum("foo")
        self.assertEqual(enum, SomeEnum.FOO)
        enum = SomeEnum("bar")
        self.assertEqual(enum, SomeEnum.BAR)

        enum = SomeEnum(SomeClass())
        self.assertEqual(enum, SomeEnum.FOO)

        enum = SomeEnum(SomeEnum.BAR)
        self.assertEqual(enum, SomeEnum.BAR)

        enum = SomeEnum(SomeEnum.LOREM)
        self.assertEqual(enum, SomeEnum.LOREM)

        enum = SomeEnum("ipsum")
        self.assertEqual(enum, SomeEnum.LOREM)

        enum = SomeEnum(OtherEnum.LOREM)
        self.assertEqual(enum, SomeEnum.LOREM)

    def test_invalid(self) -> None:
        with self.assertRaisesRegex(
            InvalidArgument,
            "^Invalid argument BAZ. Allowed values are FOO,BAR,LOREM.$",
        ):
            SomeEnum("BAZ")

        with self.assertRaisesRegex(
            ValueError,
            "^'' is not a valid SomeEnum$",
        ):
            SomeEnum("")

        with self.assertRaisesRegex(
            ValueError,
            "^None is not a valid SomeEnum$",
        ):
            SomeEnum(None)

    def test_from_string(self) -> None:
        enum = SomeEnum.from_string("FOO")
        self.assertEqual(enum, SomeEnum.FOO)
        enum = SomeEnum.from_string("BAR")
        self.assertEqual(enum, SomeEnum.BAR)

        enum = SomeEnum.from_string("foo")
        self.assertEqual(enum, SomeEnum.FOO)
        enum = SomeEnum.from_string("bar")
        self.assertEqual(enum, SomeEnum.BAR)

    def test_str(self):
        self.assertEqual(str(SomeEnum.FOO), "foo")
        self.assertEqual(str(SomeEnum.BAR), "bar")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

import httpx

from gvm.protocols.http.openvasd._vts import VtsAPI


def _mock_response(status_code=200):
    mock_response = MagicMock(spec=httpx.Response)
    mock_response.status_code = status_code
    mock_response.raise_for_status = MagicMock()
    return mock_response


class TestVtsAPI(unittest.TestCase):
    def setUp(self):
        self.mock_client = MagicMock(spec=httpx.Client)

    def test_get_all_returns_response(self):
        mock_response = _mock_response(200)
        self.mock_client.get.return_value = mock_response

        api = VtsAPI(self.mock_client)
        response = api.get_all()
        self.mock_client.get.assert_called_once_with("/vts")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(response, mock_response)

    def test_get_all_raises_httpx_error(self):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Error", request=MagicMock(), response=MagicMock(status_code=500)
        )
        api = VtsAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get_all()

    def test_get_all_returns_response_on_httpx_error_with_suppress_exceptions(
        self,
    ):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.get.return_value = mock_response

        api = VtsAPI(self.mock_client, suppress_exceptions=True)
        result = api.get_all()

        self.mock_client.get.assert_called_once_with("/vts")
        self.assertEqual(result, mock_http_error.response)

    def test_get_by_oid_returns_response(self):
        mock_response = _mock_response(200)
        self.mock_client.get.return_value = mock_response

        oid = "1.3.6.1.4.1.25623.1.0.123456"
        api = VtsAPI(self.mock_client)
        response = api.get(oid)
        self.mock_client.get.assert_called_once_with(f"/vts/{oid}")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(response, mock_response)

    def test_get_by_oid_raises_httpx_error(self):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Not Found",
            request=MagicMock(),
            response=MagicMock(status_code=404),
        )

        api = VtsAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get("nonexistent-oid")

    def test_get_by_oid_response_on_httpx_error_with_suppress_exceptions(self):
        mock_response = _mock_response()
        mock_http_error = httpx.HTTPStatusError(
            "Not Found",
            request=MagicMock(),
            response=MagicMock(status_code=404),
        )
        mock_response.raise_for_status.side_effect = mock_http_error
        self.mock_client.get.return_value = mock_response

        api = VtsAPI(self.mock_client, suppress_exceptions=True)
        response = api.get("nonexistent-oid")

        self.assertEqual(response, mock_http_error.response)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class DfnCertAdvisories:
    @staticmethod
    def get_dfn_cert_advisories(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        name: str | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of DFN-CERT Advisories

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            name: Name or identifier of the requested information
            details: Whether to include information about references to this
                information
        """
        cmd = XmlCommand("get_info")

        cmd.set_attribute("type", "DFN_CERT_ADV")

        cmd.add_filter(filter_string, filter_id)

        if name:
            cmd.set_attribute("name", name)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def get_dfn_cert_advisory(cls, cert_id: EntityID) -> Request:
        """Request a single DFN-CERT Advisory

        Args:
            cert_id: ID of an existing DFN-CERT Advisory
        """
        if not cert_id:
            raise RequiredArgument(
                function=cls.get_dfn_cert_advisory.__name__, argument="cert_id"
            )

        cmd = XmlCommand("get_info")
        cmd.set_attribute("info_id", str(cert_id))

        cmd.set_attribute("type", "DFN_CERT_ADV")

        # for single entity always request all details
        cmd.set_attribute("details", "1")

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetNvtPreferenceTestMixin:
    def test_get_preference(self):
        self.gmp.get_nvt_preference(name="foo")

        self.connection.send.has_been_called_with(
            b'<get_preferences preference="foo"/>'
        )

    def test_get_nvt_preference_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_nvt_preference(name=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_nvt_preference(name="")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_nvt_preference("")

    def test_get_nvt_preference_with_nvt_oid(self):
        self.gmp.get_nvt_preference(name="foo", nvt_oid="oid")

        self.connection.send.has_been_called_with(
            b'<get_preferences preference="foo" nvt_oid="oid"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .scan_configs import (
    GmpCloneScanConfigTestMixin,
    GmpCreateScanConfigTestMixin,
    GmpDeleteScanConfigTestMixin,
    GmpGetScanConfigsTestMixin,
    GmpGetScanConfigTestMixin,
    GmpImportScanConfigTestMixin,
    GmpModifyScanConfigSetCommentTestMixin,
    GmpModifyScanConfigSetFamilySelectionTestMixin,
    GmpModifyScanConfigSetNameTestMixin,
    GmpModifyScanConfigSetNvtPreferenceTestMixin,
    GmpModifyScanConfigSetNvtSelectionTestMixin,
    GmpModifyScanConfigSetScannerPreferenceTestMixin,
)


class Gmpv224CloneScanConfigTestCase(
    GmpCloneScanConfigTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224CreateScanConfigTestCase(
    GmpCreateScanConfigTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224DeleteScanConfigTestCase(
    GmpDeleteScanConfigTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetScanConfigTestCase(GmpGetScanConfigTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetScanConfigsTestCase(
    GmpGetScanConfigsTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ImportScanConfigTestCase(
    GmpImportScanConfigTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyScanConfigSetCommentTestCase(
    GmpModifyScanConfigSetCommentTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyScanConfigSetFamilySelectionTestCase(
    GmpModifyScanConfigSetFamilySelectionTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyScanConfigSetNvtSelectionTestCase(
    GmpModifyScanConfigSetNvtSelectionTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyScanConfigSetNameTestCase(
    GmpModifyScanConfigSetNameTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyScanConfigSetNvtPreferenceTestCase(
    GmpModifyScanConfigSetNvtPreferenceTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyScanConfigSetScannerPreferenceTestCase(
    GmpModifyScanConfigSetScannerPreferenceTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.protocols.core import Request
from gvm.protocols.gmp.requests import Version


class VersionTestCase(unittest.TestCase):
    def test_version(self) -> None:
        request = Version.get_version()

        self.assertIsInstance(request, Request)
        self.assertEqual(bytes(request), b"<get_version/>")



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpnext import GMPTestCase
from .oci_image_targets import (
    GmpCloneOCIImageTargetTestMixin,
    GmpCreateOCIImageTargetTestMixin,
    GmpDeleteOCIImageTargetTestMixin,
    GmpGetOCIImageTargetsTestMixin,
    GmpGetOCIImageTargetTestMixin,
    GmpModifyOCIImageTargetTestMixin,
)


class GmpCloneOCIImageTargetTestCase(
    GmpCloneOCIImageTargetTestMixin, GMPTestCase
):
    pass


class GmpCreateOCIImageTargetTestCase(
    GmpCreateOCIImageTargetTestMixin, GMPTestCase
):
    pass


class GmpDeleteOCIImageTargetTestCase(
    GmpDeleteOCIImageTargetTestMixin, GMPTestCase
):
    pass


class GmpGetOCIImageTargetTestCase(GmpGetOCIImageTargetsTestMixin, GMPTestCase):
    pass


class GmpGetOCIImageTargetsTestCase(GmpGetOCIImageTargetTestMixin, GMPTestCase):
    pass


class GmpModifyOCIImageTargetTestCase(
    GmpModifyOCIImageTargetTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from tests.protocols.gmpv224.entities.scanners import (
    GmpCloneScannerTestMixin,
    GmpCreateScannerTestMixin,
    GmpDeleteScannerTestMixin,
    GmpGetScannersTestMixin,
    GmpGetScannerTestMixin,
    GmpModifyScannerTestMixin,
    GmpVerifyScannerTestMixin,
)
from tests.protocols.gmpv226 import GMPTestCase


class GMPDeleteScannerTestCase(GmpDeleteScannerTestMixin, GMPTestCase):
    pass


class GMPGetScannerTestCase(GmpGetScannerTestMixin, GMPTestCase):
    pass


class GMPGetScannersTestCase(GmpGetScannersTestMixin, GMPTestCase):
    pass


class GMPCloneScannerTestCase(GmpCloneScannerTestMixin, GMPTestCase):
    pass


class GMPCreateScannerTestCase(GmpCreateScannerTestMixin, GMPTestCase):
    pass


class GMPModifyScannerTestCase(GmpModifyScannerTestMixin, GMPTestCase):
    pass


class GMPVerifyScannerTestCase(GmpVerifyScannerTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .tasks import (
    GmpCloneTaskTestMixin,
    GmpCreateContainerTaskTestMixin,
    GmpCreateTaskTestMixin,
    GmpDeleteTaskTestMixin,
    GmpGetTasksTestMixin,
    GmpGetTaskTestMixin,
    GmpModifyTaskTestMixin,
    GmpMoveTaskTestMixin,
    GmpResumeTaskTestMixin,
    GmpStartTaskTestMixin,
    GmpStopTaskTestMixin,
)


class Gmpv224CloneTaskTestCase(GmpCloneTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateContainerTaskTestCase(
    GmpCreateContainerTaskTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224CreateTaskTestCase(GmpCreateTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DeleteTaskTestCase(GmpDeleteTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTaskTestCase(GmpGetTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTasksTestCase(GmpGetTasksTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyTaskTestCase(GmpModifyTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224MoveTaskTestCase(GmpMoveTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ResumeTaskTestCase(GmpResumeTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224StartTaskTestCase(GmpStartTaskTestMixin, Gmpv224TestCase):
    pass


class Gmpv224StopTaskTestCase(GmpStopTaskTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.port_lists import (
    GmpClonePortListTestMixin,
    GmpCreatePortListTestMixin,
    GmpCreatePortRangeTestMixin,
    GmpDeletePortListTestMixin,
    GmpDeletePortRangeTestMixin,
    GmpGetPortListsTestMixin,
    GmpGetPortListTestMixin,
    GmpModifyPortListTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPClonePortListTestCase(GmpClonePortListTestMixin, GMPTestCase):
    pass


class GMPCreatePortListTestCase(GmpCreatePortListTestMixin, GMPTestCase):
    pass


class GMPCreatePortRangeListTestCase(GmpCreatePortRangeTestMixin, GMPTestCase):
    pass


class GMPDeletePortListTestCase(GmpDeletePortListTestMixin, GMPTestCase):
    pass


class GMPDeletePortRangeTestCase(GmpDeletePortRangeTestMixin, GMPTestCase):
    pass


class GMPGetPortListTestCase(GmpGetPortListTestMixin, GMPTestCase):
    pass


class GMPGetPortListsTestCase(GmpGetPortListsTestMixin, GMPTestCase):
    pass


class GMPModifyPortListTestCase(GmpModifyPortListTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpRestoreFromTrashcanTestMixin:
    def test_restore_from_trashcan(self):
        self.gmp.restore_from_trashcan("a1")

        self.connection.send.has_been_called_with(b'<restore id="a1"/>')

    def test_restore_from_trashcan_missing_id(self):
        with self.assertRaises(GvmError):
            self.gmp.restore_from_trashcan(None)

        with self.assertRaises(GvmError):
            self.gmp.restore_from_trashcan("")



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later

from collections.abc import Mapping, Sequence
from typing import Any

from gvm.protocols.gmp.requests import EntityID

from ...utils import SupportsStr
from .._protocol import T
from ._gmp227 import GMPv227
from .requests.next import (
    AgentGroups,
    AgentInstallers,
    Agents,
    Credentials,
    CredentialStoreCredentialType,
    CredentialStores,
    IntegrationConfigs,
    OCIImageTargets,
    Tasks,
)
from .requests.v224 import HostsOrdering


class GMPNext(GMPv227[T]):
    """
    A class implementing the "Next" version of Greenbone Management Protocol (GMP)
    containing features that are not part of the stable release yet.

    These features may change at any time and may not be available in all builds
    of the gvmd back-end.

    Example:

        .. code-block:: python

            from gvm.protocols.gmp.next import GMP

            with GMP(connection) as gmp:
                resp = gmp.get_tasks()
    """

    @staticmethod
    def get_protocol_version() -> tuple[int, int]:
        return (22, 8)

    def get_agent_installers(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        details: bool | None = None,
    ) -> T:
        """Request a list of agent installers

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan agent installers instead
            details: Whether to include extra details like tasks using this
                scanner
        """
        return self._send_request_and_transform_response(
            AgentInstallers.get_agent_installers(
                filter_string=filter_string,
                filter_id=filter_id,
                trash=trash,
                details=details,
            )
        )

    def get_agent_installer(self, agent_installer_id: EntityID) -> T:
        """Request a single agent installer

        Args:
            agent_installer_id: UUID of an existing agent installer
        """
        return self._send_request_and_transform_response(
            AgentInstallers.get_agent_installer(agent_installer_id)
        )

    def get_agent_installer_file(self, agent_installer_id: EntityID) -> T:
        """Request a single agent installer file

        Args:
            agent_installer_id: UUID of an existing agent installer
        """
        return self._send_request_and_transform_response(
            AgentInstallers.get_agent_installer_file(agent_installer_id)
        )

    def get_agents(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
    ) -> T:
        """Request a list of agents.

        Args:
            filter_string: Filter term to use for the query.
            filter_id: UUID of an existing filter to use for the query.
            details: Whether to include detailed agent info.
        """
        return self._send_request_and_transform_response(
            Agents.get_agents(
                filter_string=filter_string,
                filter_id=filter_id,
                details=details,
            )
        )

    def modify_agents(
        self,
        agent_ids: list[EntityID],
        *,
        authorized: bool | None = None,
        update_to_latest: bool | None = None,
        config: Mapping[str, Any] | None = None,
        comment: str | None = None,
    ) -> T:
        """
        Modify multiple agents.

        Args:
            agent_ids: List of agent UUIDs to modify.
            authorized: Whether the agent is authorized.
            update_to_latest: Whether the agent is allowed to update to latest automatically.
            config: Nested config for Agent Controller.
            comment: Optional comment for the change.
        """
        return self._send_request_and_transform_response(
            Agents.modify_agents(
                agent_ids=agent_ids,
                authorized=authorized,
                update_to_latest=update_to_latest,
                config=config,
                comment=comment,
            )
        )

    def delete_agents(self, agent_ids: list[EntityID]) -> T:
        """Delete multiple agents

        Args:
            agent_ids: List of agent UUIDs to delete
        """
        return self._send_request_and_transform_response(
            Agents.delete_agents(agent_ids=agent_ids)
        )

    def modify_agent_control_scan_config(
        self,
        agent_control_id: EntityID,
        config: Mapping[str, Any],
    ) -> T:
        """
        Modify agent control scan config.

        Args:
            agent_control_id: The agent control UUID.
            config: Nested config for Agent Controller.
        """
        return self._send_request_and_transform_response(
            Agents.modify_agent_control_scan_config(
                agent_control_id=agent_control_id,
                config_defaults=config,
            )
        )

    def get_agent_groups(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
    ) -> T:
        """Request a list of agent groups.

        Args:
            filter_string: Filter expression to use.
            filter_id: UUID of a predefined filter.
            trash: If True, return trashed agent groups.

        Returns:
            Request object to fetch agent groups.
        """
        return self._send_request_and_transform_response(
            AgentGroups.get_agent_groups(
                filter_string=filter_string,
                filter_id=filter_id,
                trash=trash,
            )
        )

    def get_agent_group(self, agent_group_id: EntityID) -> T:
        """Request a single agent group by ID.

        Args:
            agent_group_id: UUID of the agent group.

        Raises:
            RequiredArgument: If agent_group_id is not provided.

        Returns:
            Request object to fetch the specific agent group.
        """
        return self._send_request_and_transform_response(
            AgentGroups.get_agent_group(
                agent_group_id=agent_group_id,
            )
        )

    def create_agent_group(
        self,
        name: str,
        agent_ids: list[str],
        *,
        comment: str | None = None,
    ) -> T:
        """Create a new agent group.

        Args:
            name: Name of the new agent group.
            agent_ids: List of agent UUIDs to include in the group (required).
            comment: Optional comment for the group.

        Raises:
            RequiredArgument: If name or agent_ids is not provided.
        """
        return self._send_request_and_transform_response(
            AgentGroups.create_agent_group(
                name=name,
                comment=comment,
                agent_ids=agent_ids,
            )
        )

    def modify_agent_group(
        self,
        agent_group_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        agent_ids: list[str] | None = None,
    ) -> T:
        """Modify an existing agent group.

        Args:
            agent_group_id: UUID of the group to modify.
            name: Optional new name for the group.
            comment: Optional comment for the group.
            agent_ids: Optional list of agent UUIDs to set for the group.

        Raises:
            RequiredArgument: If agent_group_id is not provided.
        """
        return self._send_request_and_transform_response(
            AgentGroups.modify_agent_group(
                agent_group_id=agent_group_id,
                name=name,
                comment=comment,
                agent_ids=agent_ids,
            )
        )

    def delete_agent_group(
        self,
        agent_group_id: EntityID,
        ultimate: bool = False,
    ) -> T:
        """Delete an existing agent group.

        Args:
            agent_group_id: UUID of the group to delete.
            ultimate: Whether to permanently delete or move to trashcan.

        Raises:
            RequiredArgument: If agent_group_id is not provided.
        """
        return self._send_request_and_transform_response(
            AgentGroups.delete_agent_group(
                agent_group_id=agent_group_id,
                ultimate=ultimate,
            )
        )

    def clone_agent_group(
        self,
        agent_group_id: EntityID,
    ) -> T:
        """Clone an existing agent group

        Args:
            agent_group_id: UUID of an existing agent group to clone from

        Returns:
            Request: GMP command to create a new agent group based on a copy
        """
        return self._send_request_and_transform_response(
            AgentGroups.clone_agent_group(agent_group_id)
        )

    def sync_agents(self) -> T:
        """Trigger agents synchronization from all agent controllers."""
        return self._send_request_and_transform_response(Agents.sync_agents())

    def create_credential_store_credential(
        self,
        name: str,
        credential_type: CredentialStoreCredentialType | str,
        *,
        comment: str | None = None,
        credential_store_id: EntityID | None = None,
        vault_id: str | None = None,
        host_identifier: str | None = None,
    ) -> T:
        """Create a new credential store type credential

        Args:
            name: Name of the credential
            credential_type: Type of the credential
            comment: Optional comment for the credential object
            credential_store_id: Optional credential store id to fetch the credential from
            vault_id: Vault id used to fetch the credential from credential store
            host_identifier: Host identifier used to fetch the credential from credential store
        """
        return self._send_request_and_transform_response(
            Credentials.create_credential_store_credential(
                name=name,
                credential_type=credential_type,
                comment=comment,
                credential_store_id=credential_store_id,
                vault_id=vault_id,
                host_identifier=host_identifier,
            )
        )

    def modify_credential_store_credential(
        self,
        credential_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        credential_store_id: EntityID | None = None,
        vault_id: str | None = None,
        host_identifier: str | None = None,
    ) -> T:
        """Modify an existing credential stored in a credential store

        Args:
            credential_id: UUID of the credential to modify
            name: Name of the credential
            comment: Optional comment for the credential object
            credential_store_id: Optional credential store id to fetch the credential from
            vault_id: Vault id used to fetch the credential from credential store
            host_identifier: Host identifier used to fetch the credential from credential store
        """
        return self._send_request_and_transform_response(
            Credentials.modify_credential_store_credential(
                credential_id=credential_id,
                name=name,
                comment=comment,
                credential_store_id=credential_store_id,
                vault_id=vault_id,
                host_identifier=host_identifier,
            )
        )

    def get_credential_store(
        self,
        credential_store_id: EntityID,
        *,
        details: bool | None = None,
    ) -> T:
        """Request a credential store

        Args:
            credential_store_id: ID of credential store to fetch
            details: True to request all details
        """
        return self._send_request_and_transform_response(
            CredentialStores.get_credential_store(
                credential_store_id=credential_store_id,
                details=details,
            )
        )

    def get_credential_stores(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
    ) -> T:
        """Request a list of credential stores

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            details: True to request all details
        """
        return self._send_request_and_transform_response(
            CredentialStores.get_credential_stores(
                filter_string=filter_string,
                filter_id=filter_id,
                details=details,
            )
        )

    def modify_credential_store(
        self,
        credential_store_id: EntityID,
        *,
        active: bool | None = None,
        host: str | None = None,
        port: int | None = None,
        path: str | None = None,
        app_id: str | None = None,
        client_cert: str | None = None,
        client_key: str | None = None,
        client_pkcs12_file: str | None = None,
        passphrase: str | None = None,
        server_ca_cert: str | None = None,
        comment: str | None = None,
    ) -> T:
        """Modify an existing credential store

        Args:
            credential_store_id: ID of credential store to fetch
            active: Whether the credential store is active
            host: The host to use for reaching the credential store
            port: The port to use for reaching the credential store
            path: The URI path the credential store is using
            app_id: Depends on the credential store used. Usually called the same in the credential store
            client_cert: The client certificate to use for authorization, as a plain string
            client_key: The client key to use for authorization, as a plain string
            client_pkcs12_file: The pkcs12 file contents to use for authorization, as a plain string
                (alternative to using client_cert and client_key)
            passphrase: The passphrase to use to decrypt client_pkcs12_file or client_key file
            server_ca_cert: The server certificate, so the credential store can be trusted
            comment: An optional comment to store alongside the credential store
        """
        return self._send_request_and_transform_response(
            CredentialStores.modify_credential_store(
                credential_store_id=credential_store_id,
                active=active,
                host=host,
                port=port,
                path=path,
                app_id=app_id,
                client_cert=client_cert,
                client_key=client_key,
                client_pkcs12_file=client_pkcs12_file,
                passphrase=passphrase,
                server_ca_cert=server_ca_cert,
                comment=comment,
            )
        )

    def verify_credential_store(
        self,
        credential_store_id: EntityID,
    ) -> T:
        """Verify that the connection to an existing credential store works

        Args:
            credential_store_id: The uuid of the credential store to verify
        """
        return self._send_request_and_transform_response(
            CredentialStores.verify_credential_store(
                credential_store_id=credential_store_id,
            )
        )

    def create_oci_image_target(
        self,
        name: str,
        image_references: list[str],
        *,
        comment: str | None = None,
        credential_id: EntityID | None = None,
    ) -> T:
        """Create a new OCI image target

        Args:
            name: Name of the OCI image target
            image_references: List of OCI image URLs to scan
            comment: Comment for the target
            credential_id: UUID of a credential to use on target
        """
        return self._send_request_and_transform_response(
            OCIImageTargets.create_oci_image_target(
                name=name,
                image_references=image_references,
                comment=comment,
                credential_id=credential_id,
            )
        )

    def modify_oci_image_target(
        self,
        oci_image_target_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        image_references: list[str] | None = None,
        credential_id: EntityID | None = None,
    ) -> T:
        """Modify an existing OCI image target.

        Args:
            oci_image_target_id: UUID of target to modify.
            comment: Comment on target.
            name: Name of target.
            image_references: List of OCI image URLs to scan.
            credential_id: UUID of credential to use on target.
        """
        return self._send_request_and_transform_response(
            OCIImageTargets.modify_oci_image_target(
                oci_image_target_id,
                name=name,
                comment=comment,
                image_references=image_references,
                credential_id=credential_id,
            )
        )

    def clone_oci_image_target(self, oci_image_target_id: EntityID) -> T:
        """Clone an existing OCI image target.

        Args:
            oci_image_target_id: UUID of an existing OCI image target to clone.
        """
        return self._send_request_and_transform_response(
            OCIImageTargets.clone_oci_image_target(oci_image_target_id)
        )

    def delete_oci_image_target(
        self, oci_image_target_id: EntityID, *, ultimate: bool | None = False
    ) -> T:
        """Delete an existing OCI image target.

        Args:
            oci_image_target_id: UUID of an existing OCI image target to delete.
            ultimate: Whether to remove entirely or to the trashcan.
        """
        return self._send_request_and_transform_response(
            OCIImageTargets.delete_oci_image_target(
                oci_image_target_id, ultimate=ultimate
            )
        )

    def get_oci_image_target(
        self, oci_image_target_id: EntityID, *, tasks: bool | None = None
    ) -> T:
        """Request a single OCI image target.

        Args:
            oci_image_target_id: UUID of the OCI image target to request.
            tasks: Whether to include list of tasks that use the target
        """
        return self._send_request_and_transform_response(
            OCIImageTargets.get_oci_image_target(
                oci_image_target_id, tasks=tasks
            )
        )

    def get_oci_image_targets(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        tasks: bool | None = None,
    ) -> T:
        """Request a list of OCI image targets.

        Args:
            filter_string: Filter term to use for the query.
            filter_id: UUID of an existing filter to use for the query.
            trash: Whether to include targets in the trashcan.
            tasks: Whether to include list of tasks that use the target.
        """
        return self._send_request_and_transform_response(
            OCIImageTargets.get_oci_image_targets(
                filter_string=filter_string,
                filter_id=filter_id,
                trash=trash,
                tasks=tasks,
            )
        )

    def clone_task(self, task_id: EntityID) -> T:
        """Clone an existing task

        Args:
            task_id: UUID of existing task to clone from
        """
        return self._send_request_and_transform_response(
            Tasks.clone_task(task_id)
        )

    def create_agent_group_task(
        self,
        name: str,
        agent_group_id: EntityID,
        scanner_id: EntityID,
        *,
        comment: str | None = None,
        alterable: bool | None = None,
        schedule_id: EntityID | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        schedule_periods: int | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> T:
        """Create a new scan task using an agent group.

        Args:
            name: Name of the new task.
            agent_group_id: UUID of the agent group to be scanned.
            scanner_id: UUID of scanner to use for scanning the agents.
            comment: Optional comment for the task.
            alterable: Whether the task should be alterable.
            alert_ids: List of UUIDs for alerts to be applied to the task.
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: Limit to number of scheduled runs, 0 for unlimited.
            observers: List of usernames or IDs allowed to observe the task.
            preferences: Scanner preferences as name/value pairs.
        """
        return self._send_request_and_transform_response(
            Tasks.create_agent_group_task(
                name=name,
                agent_group_id=agent_group_id,
                scanner_id=scanner_id,
                comment=comment,
                alterable=alterable,
                schedule_id=schedule_id,
                alert_ids=alert_ids,
                schedule_periods=schedule_periods,
                observers=observers,
                preferences=preferences,
            )
        )

    def create_container_image_task(
        self,
        name: str,
        oci_image_target_id: EntityID,
        scanner_id: EntityID,
        *,
        comment: str | None = None,
        alterable: bool | None = None,
        schedule_id: EntityID | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        schedule_periods: int | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> T:
        """Create a new scan task using an OCI image target.

        Args:
            name: Name of the new task.
            oci_image_target_id: UUID of the OCI image target to be scanned.
            scanner_id: UUID of scanner to use for scanning the agents.
            comment: Optional comment for the task.
            alterable: Whether the task should be alterable.
            alert_ids: List of UUIDs for alerts to be applied to the task.
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: Limit to number of scheduled runs, 0 for unlimited.
            observers: List of usernames or IDs allowed to observe the task.
            preferences: Scanner preferences as name/value pairs.
        """
        return self._send_request_and_transform_response(
            Tasks.create_container_image_task(
                name=name,
                oci_image_target_id=oci_image_target_id,
                scanner_id=scanner_id,
                comment=comment,
                alterable=alterable,
                schedule_id=schedule_id,
                alert_ids=alert_ids,
                schedule_periods=schedule_periods,
                observers=observers,
                preferences=preferences,
            )
        )

    def create_import_task(self, name: str, *, comment: str | None = None) -> T:
        """Create a new import task

        An import task is a "meta" task to import and view reports from other
        systems.

        Args:
            name: Name of the task
            comment: Comment for the task
        """
        return self._send_request_and_transform_response(
            Tasks.create_import_task(name=name, comment=comment)
        )

    def create_container_task(
        self, name: str, *, comment: str | None = None
    ) -> T:
        """[DEPRECATED] Use create_import_task instead.

        This method will be removed in a future version.
        """
        return self._send_request_and_transform_response(
            Tasks.create_container_task(name=name, comment=comment)
        )

    def create_task(
        self,
        name: str,
        config_id: EntityID,
        target_id: EntityID,
        scanner_id: EntityID,
        *,
        alterable: bool | None = None,
        hosts_ordering: HostsOrdering | None = None,
        schedule_id: EntityID | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        comment: str | None = None,
        schedule_periods: int | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> T:
        """Create a new scan task

        Args:
            name: Name of the new task
            config_id: UUID of config to use by the task
            target_id: UUID of target to be scanned
            scanner_id: UUID of scanner to use for scanning the target
            comment: Comment for the task
            alterable: Whether the task should be alterable
            alert_ids: List of UUIDs for alerts to be applied to the task
            hosts_ordering: The order hosts are scanned in
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: A limit to the number of times the task will be
                scheduled, or 0 for no limit
            observers: List of names or ids of users which should be allowed to
                observe this task
            preferences: Name/Value pairs of scanner preferences.
        """
        return self._send_request_and_transform_response(
            Tasks.create_task(
                name=name,
                config_id=config_id,
                target_id=target_id,
                scanner_id=scanner_id,
                alterable=alterable,
                hosts_ordering=hosts_ordering,
                schedule_id=schedule_id,
                alert_ids=alert_ids,
                comment=comment,
                schedule_periods=schedule_periods,
                observers=observers,
                preferences=preferences,
            )
        )

    def delete_task(
        self, task_id: EntityID, *, ultimate: bool | None = False
    ) -> T:
        """Deletes an existing task

        Args:
            task_id: UUID of the task to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        return self._send_request_and_transform_response(
            Tasks.delete_task(task_id=task_id, ultimate=ultimate)
        )

    def get_tasks(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        details: bool | None = None,
        schedules_only: bool | None = None,
        ignore_pagination: bool | None = None,
    ) -> T:
        """Request a list of tasks

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan tasks instead
            details: Whether to include full task details
            schedules_only: Whether to only include id, name and schedule
                details
            ignore_pagination: Whether to ignore pagination settings (filter
                terms "first" and "rows"). Default is False.
        """
        return self._send_request_and_transform_response(
            Tasks.get_tasks(
                filter_string=filter_string,
                filter_id=filter_id,
                trash=trash,
                details=details,
                schedules_only=schedules_only,
                ignore_pagination=ignore_pagination,
            )
        )

    def get_task(self, task_id: EntityID) -> T:
        """Request a single task

        Args:
            task_id: UUID of an existing task
        """
        return self._send_request_and_transform_response(
            Tasks.get_task(task_id=task_id)
        )

    def modify_task(
        self,
        task_id: EntityID,
        *,
        name: str | None = None,
        config_id: EntityID | None = None,
        target_id: EntityID | None = None,
        scanner_id: EntityID | None = None,
        agent_group_id: EntityID | None = None,
        oci_image_target_id: EntityID | None = None,
        alterable: bool | None = None,
        hosts_ordering: HostsOrdering | None = None,
        schedule_id: EntityID | None = None,
        schedule_periods: int | None = None,
        comment: str | None = None,
        alert_ids: Sequence[EntityID] | None = None,
        observers: Sequence[str] | None = None,
        preferences: Mapping[str, SupportsStr] | None = None,
    ) -> T:
        """Modifies an existing task.

        Args:
            task_id: UUID of task to modify.
            name: The name of the task.
            config_id: UUID of scan config to use by the task
            target_id: UUID of target to be scanned
            scanner_id: UUID of scanner to use for scanning the target
            agent_group_id: UUID of agent group to use for scanning
            oci_image_target_id: UUID of the OCI Image target to be scanned.
            comment: The comment on the task.
            alert_ids: List of UUIDs for alerts to be applied to the task
            hosts_ordering: The order hosts are scanned in
            schedule_id: UUID of a schedule when the task should be run.
            schedule_periods: A limit to the number of times the task will be
                scheduled, or 0 for no limit.
            observers: List of names or ids of users which should be allowed to
                observe this task
            preferences: Name/Value pairs of scanner preferences.
        """
        return self._send_request_and_transform_response(
            Tasks.modify_task(
                task_id=task_id,
                name=name,
                config_id=config_id,
                target_id=target_id,
                scanner_id=scanner_id,
                agent_group_id=agent_group_id,
                oci_image_target_id=oci_image_target_id,
                alterable=alterable,
                hosts_ordering=hosts_ordering,
                schedule_id=schedule_id,
                alert_ids=alert_ids,
                comment=comment,
                schedule_periods=schedule_periods,
                observers=observers,
                preferences=preferences,
            )
        )

    def move_task(
        self, task_id: EntityID, *, slave_id: EntityID | None = None
    ) -> T:
        """Move an existing task to another GMP slave scanner or the master

        Args:
            task_id: UUID of the task to be moved
            slave_id: UUID of the sensor to reassign the task to, empty for master.
        """
        return self._send_request_and_transform_response(
            Tasks.move_task(
                task_id=task_id,
                slave_id=slave_id,
            )
        )

    def start_task(self, task_id: EntityID) -> T:
        """Start an existing task

        Args:
            task_id: UUID of the task to be started
        """
        return self._send_request_and_transform_response(
            Tasks.start_task(task_id=task_id)
        )

    def resume_task(self, task_id: EntityID) -> T:
        """Resume an existing stopped task

        Args:
            task_id: UUID of the task to be resumed
        """
        return self._send_request_and_transform_response(
            Tasks.resume_task(task_id=task_id)
        )

    def stop_task(self, task_id: EntityID) -> T:
        """Stop an existing running task

        Args:
            task_id: UUID of the task to be stopped
        """
        return self._send_request_and_transform_response(
            Tasks.stop_task(task_id=task_id)
        )

    def get_integration_config(
        self, integration_config_id: EntityID, *, details: bool | None = None
    ) -> T:
        """Request a single Integration Configuration.

        Args:
           integration_config_id: UUID of the integration config to request.
           details: Whether to include detail information.
        """
        return self._send_request_and_transform_response(
            IntegrationConfigs.get_integration_config(
                integration_config_id=integration_config_id, details=details
            )
        )

    def get_integration_configs(
        self,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
    ) -> T:
        """Request a list of Integration Configurations.

        Args:
            filter_string: Filter term to use for the query.
            filter_id: UUID of an existing filter to use for the query.
        """
        return self._send_request_and_transform_response(
            IntegrationConfigs.get_integration_configs(
                filter_string=filter_string,
                filter_id=filter_id,
            )
        )

    def modify_integration_config(
        self,
        integration_config_id: EntityID,
        *,
        service_url: str | None = None,
        service_cacert: str | None = None,
        oidc_provider_url: str | None = None,
        oidc_provider_client_id: str | None = None,
        oidc_provider_client_secret: str | None = None,
    ) -> T:
        """Modify an existing Integration Configuration.

        Args:
            integration_config_id: UUID of configuration to modify.
            service_url: Integration Service URL.
            service_cacert: Integration Service Certificate.
            oidc_provider_url: OIDC Provider URL.
            oidc_provider_client_id: OIDC Provider Client ID.
            oidc_provider_client_secret: OIDC Provider Client Secret.
        """
        return self._send_request_and_transform_response(
            IntegrationConfigs.modify_integration_config(
                integration_config_id=integration_config_id,
                service_url=service_url,
                service_cacert=service_cacert,
                oidc_provider_url=oidc_provider_url,
                oidc_provider_client_id=oidc_provider_client_id,
                oidc_provider_client_secret=oidc_provider_client_secret,
            )
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v225 import AggregateStatistic


class GetAggregateStatisticFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            AggregateStatistic.from_string("foo")

    def test_none_or_empty(self):
        ct = AggregateStatistic.from_string(None)
        self.assertIsNone(ct)
        ct = AggregateStatistic.from_string("")
        self.assertIsNone(ct)

    def test_count(self):
        ct = AggregateStatistic.from_string("count")
        self.assertEqual(ct, AggregateStatistic.COUNT)

    def test_c_count(self):
        ct = AggregateStatistic.from_string("c_count")
        self.assertEqual(ct, AggregateStatistic.C_COUNT)

    def test_c_sum(self):
        ct = AggregateStatistic.from_string("c_sum")
        self.assertEqual(ct, AggregateStatistic.C_SUM)

    def test_max(self):
        ct = AggregateStatistic.from_string("max")
        self.assertEqual(ct, AggregateStatistic.MAX)

    def test_mean(self):
        ct = AggregateStatistic.from_string("mean")
        self.assertEqual(ct, AggregateStatistic.MEAN)

    def test_min(self):
        ct = AggregateStatistic.from_string("min")
        self.assertEqual(ct, AggregateStatistic.MIN)

    def test_sum(self):
        ct = AggregateStatistic.from_string("sum")
        self.assertEqual(ct, AggregateStatistic.SUM)

    def test_text(self):
        ct = AggregateStatistic.from_string("text")
        self.assertEqual(ct, AggregateStatistic.TEXT)

    def test_value(self):
        ct = AggregateStatistic.from_string("value")
        self.assertEqual(ct, AggregateStatistic.VALUE)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v225 import SortOrder


class GetSortOrderFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            SortOrder.from_string("foo")

    def test_none_or_empty(self):
        ct = SortOrder.from_string(None)
        self.assertIsNone(ct)
        ct = SortOrder.from_string("")
        self.assertIsNone(ct)

    def test_ascending(self):
        ct = SortOrder.from_string("ascending")
        self.assertEqual(ct, SortOrder.ASCENDING)

    def test_descending(self):
        ct = SortOrder.from_string("descending")
        self.assertEqual(ct, SortOrder.DESCENDING)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_schedule import GmpCloneScheduleTestMixin
from .test_create_schedule import GmpCreateScheduleTestMixin
from .test_delete_schedule import GmpDeleteScheduleTestMixin
from .test_get_schedule import GmpGetScheduleTestMixin
from .test_get_schedules import GmpGetSchedulesTestMixin
from .test_modify_schedule import GmpModifyScheduleTestMixin

__all__ = (
    "GmpCloneScheduleTestMixin",
    "GmpCreateScheduleTestMixin",
    "GmpDeleteScheduleTestMixin",
    "GmpGetScheduleTestMixin",
    "GmpGetSchedulesTestMixin",
    "GmpModifyScheduleTestMixin",
)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_group import GmpCloneGroupTestMixin
from .test_create_group import GmpCreateGroupTestMixin
from .test_delete_group import GmpDeleteGroupTestMixin
from .test_get_group import GmpGetGroupTestMixin
from .test_get_groups import GmpGetGroupsTestMixin
from .test_modify_group import GmpModifyGroupTestMixin

__all__ = (
    "GmpCloneGroupTestMixin",
    "GmpCreateGroupTestMixin",
    "GmpDeleteGroupTestMixin",
    "GmpGetGroupTestMixin",
    "GmpGetGroupsTestMixin",
    "GmpModifyGroupTestMixin",
)



# SPDX-FileCopyrightText: 2026 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpnext import GMPTestCase
from .integration_configs.test_get_integration_config import (
    GmpGetIntegrationConfigTestMixin,
)
from .integration_configs.test_get_integration_configs import (
    GmpGetIntegrationConfigsTestMixin,
)
from .integration_configs.test_modify_integration_config import (
    GmpModifyIntegrationConfigTestMixin,
)


class GmpGetIntegrationConfigTestCase(
    GmpGetIntegrationConfigTestMixin, GMPTestCase
):
    pass


class GmpGetIntegrationConfigsTestCase(
    GmpGetIntegrationConfigsTestMixin, GMPTestCase
):
    pass


class GmpModifyIntegrationConfigTestCase(
    GmpModifyIntegrationConfigTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetUserSettingTestMixin:
    def test_get_setting_simple(self):
        self.gmp.get_user_setting("id")

        self.connection.send.has_been_called_with(
            b'<get_settings setting_id="id"/>'
        )

    def test_get_setting_missing_setting_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_user_setting(setting_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_user_setting("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v224 import Help, HelpFormat


class HelpTestCase(unittest.TestCase):
    def test_help(self):
        request = Help.help()

        self.assertEqual(bytes(request), b'<help type=""/>')

    def test_help_type_brief(self):
        request = Help.help(brief=True)

        self.assertEqual(bytes(request), b'<help type="brief"/>')

    def test_invalid_help_format(self):
        with self.assertRaises(InvalidArgument):
            Help.help(help_format="foo")

    def test_html_format(self):
        request = Help.help(help_format=HelpFormat.HTML)

        self.assertEqual(bytes(request), b'<help type="" format="html"/>')



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.targets import (
    GmpCloneTargetTestMixin,
    GmpCreateTargetTestMixin,
    GmpDeleteTargetTestMixin,
    GmpGetTargetsTestMixin,
    GmpGetTargetTestMixin,
    GmpModifyTargetTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPCloneTargetTestCase(GmpCloneTargetTestMixin, GMPTestCase):
    pass


class GMPCreateTargetTestCase(GmpCreateTargetTestMixin, GMPTestCase):
    pass


class GMPDeleteTargetTestCase(GmpDeleteTargetTestMixin, GMPTestCase):
    pass


class GMPGetTargetTestCase(GmpGetTargetTestMixin, GMPTestCase):
    pass


class GMPGetTargetsTestCase(GmpGetTargetsTestMixin, GMPTestCase):
    pass


class GMPModifyTargetTestCase(GmpModifyTargetTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v226 import Filters, FilterType


class FilterTestsCase(unittest.TestCase):
    def test_clone_filter(self):
        request = Filters.clone_filter("filter_id")
        self.assertEqual(
            bytes(request),
            b"<create_filter><copy>filter_id</copy></create_filter>",
        )

    def test_clone_filter_missing_filter_id(self):
        with self.assertRaises(RequiredArgument):
            Filters.clone_filter(None)

        with self.assertRaises(RequiredArgument):
            Filters.clone_filter("")

    def test_create_filter(self):
        request = Filters.create_filter("filter_name")
        self.assertEqual(
            bytes(request),
            b"<create_filter><name>filter_name</name></create_filter>",
        )

    def test_create_filter_with_comment(self):
        request = Filters.create_filter("filter_name", comment="comment")
        self.assertEqual(
            bytes(request),
            b"<create_filter>"
            b"<name>filter_name</name>"
            b"<comment>comment</comment>"
            b"</create_filter>",
        )

    def test_create_filter_with_term(self):
        request = Filters.create_filter("filter_name", term="term")
        self.assertEqual(
            bytes(request),
            b"<create_filter>"
            b"<name>filter_name</name>"
            b"<term>term</term>"
            b"</create_filter>",
        )

    def test_create_filter_with_filter_type(self):
        request = Filters.create_filter(
            "filter_name", filter_type=FilterType.ALERT
        )
        self.assertEqual(
            bytes(request),
            b"<create_filter>"
            b"<name>filter_name</name>"
            b"<type>alert</type>"
            b"</create_filter>",
        )

        request = Filters.create_filter(
            "filter_name",
            filter_type="alert",
        )
        self.assertEqual(
            bytes(request),
            b"<create_filter>"
            b"<name>filter_name</name>"
            b"<type>alert</type>"
            b"</create_filter>",
        )

    def test_create_filter_invalid_filter_type(self):
        with self.assertRaises(InvalidArgument):
            Filters.create_filter("filter_name", filter_type="invalid")

    def test_create_filter_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Filters.create_filter(None)

        with self.assertRaises(RequiredArgument):
            Filters.create_filter("")

    def test_delete_filter(self):
        request = Filters.delete_filter("filter_id")
        self.assertEqual(
            bytes(request),
            b'<delete_filter filter_id="filter_id" ultimate="0"/>',
        )

    def test_delete_filter_with_ultimate(self):
        request = Filters.delete_filter("filter_id", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_filter filter_id="filter_id" ultimate="1"/>',
        )

        request = Filters.delete_filter("filter_id", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_filter filter_id="filter_id" ultimate="0"/>',
        )

    def test_delete_filter_missing_filter_id(self):
        with self.assertRaises(RequiredArgument):
            Filters.delete_filter(None)

        with self.assertRaises(RequiredArgument):
            Filters.delete_filter("")

    def test_get_filters(self):
        request = Filters.get_filters()
        self.assertEqual(
            bytes(request),
            b"<get_filters/>",
        )

    def test_get_filters_with_filter_string(self):
        request = Filters.get_filters(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_filters filter="filter_string"/>',
        )

    def test_get_filters_with_filter_id(self):
        request = Filters.get_filters(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_filters filt_id="filter_id"/>',
        )

    def test_get_filters_with_trash(self):
        request = Filters.get_filters(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_filters trash="1"/>',
        )

        request = Filters.get_filters(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_filters trash="0"/>',
        )

    def test_get_filters_with_alerts(self):
        request = Filters.get_filters(alerts=True)
        self.assertEqual(
            bytes(request),
            b'<get_filters alerts="1"/>',
        )

        request = Filters.get_filters(alerts=False)
        self.assertEqual(
            bytes(request),
            b'<get_filters alerts="0"/>',
        )

    def test_get_filter(self):
        request = Filters.get_filter("filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_filters filter_id="filter_id"/>',
        )

    def test_get_filter_with_alerts(self):
        request = Filters.get_filter("filter_id", alerts=True)
        self.assertEqual(
            bytes(request),
            b'<get_filters filter_id="filter_id" alerts="1"/>',
        )

        request = Filters.get_filter("filter_id", alerts=False)
        self.assertEqual(
            bytes(request),
            b'<get_filters filter_id="filter_id" alerts="0"/>',
        )

    def test_get_filter_missing_filter_id(self):
        with self.assertRaises(RequiredArgument):
            Filters.get_filter(None)

        with self.assertRaises(RequiredArgument):
            Filters.get_filter("")

    def test_modify_filter(self):
        request = Filters.modify_filter("filter_id")
        self.assertEqual(
            bytes(request),
            b'<modify_filter filter_id="filter_id"/>',
        )

    def test_modify_filter_with_comment(self):
        request = Filters.modify_filter("filter_id", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_filter filter_id="filter_id">'
            b"<comment>comment</comment>"
            b"</modify_filter>",
        )

    def test_modify_filter_with_name(self):
        request = Filters.modify_filter("filter_id", name="name")
        self.assertEqual(
            bytes(request),
            b'<modify_filter filter_id="filter_id">'
            b"<name>name</name>"
            b"</modify_filter>",
        )

    def test_modify_filter_with_term(self):
        request = Filters.modify_filter("filter_id", term="term")
        self.assertEqual(
            bytes(request),
            b'<modify_filter filter_id="filter_id">'
            b"<term>term</term>"
            b"</modify_filter>",
        )

    def test_modify_filter_with_type(self):
        request = Filters.modify_filter(
            "filter_id", filter_type=FilterType.ALERT
        )
        self.assertEqual(
            bytes(request),
            b'<modify_filter filter_id="filter_id">'
            b"<type>alert</type>"
            b"</modify_filter>",
        )

        request = Filters.modify_filter("filter_id", filter_type="alert")
        self.assertEqual(
            bytes(request),
            b'<modify_filter filter_id="filter_id">'
            b"<type>alert</type>"
            b"</modify_filter>",
        )

    def test_modify_filter_missing_filter_id(self):
        with self.assertRaises(RequiredArgument):
            Filters.modify_filter(None)

        with self.assertRaises(RequiredArgument):
            Filters.modify_filter("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class OperatingSystems:
    @classmethod
    def delete_operating_system(
        cls,
        operating_system_id: EntityID,
    ) -> Request:
        """Deletes an existing operating system

        Args:
            operating_system_id: UUID of the single operating_system to delete.
        """
        if not operating_system_id:
            raise RequiredArgument(
                function=cls.delete_operating_system.__name__,
                argument="operating_system_id",
            )

        cmd = XmlCommand("delete_asset")
        cmd.set_attribute("asset_id", str(operating_system_id))

        return cmd

    @staticmethod
    def get_operating_systems(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of operating systems

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            details: Whether to include additional information (e.g. tags)
        """

        cmd = XmlCommand("get_assets")

        cmd.set_attribute("type", "os")

        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def get_operating_system(
        cls, operating_system_id: EntityID, *, details: bool | None = None
    ) -> Request:
        """Request a single operating system

        Args:
            operating_system_id: UUID of an existing operating_system
            details: Whether to include additional information (e.g. tags)
        """
        cmd = XmlCommand("get_assets")

        if not operating_system_id:
            raise RequiredArgument(
                function=cls.get_operating_system.__name__,
                argument="operating_system_id",
            )

        cmd.set_attribute("asset_id", str(operating_system_id))
        cmd.set_attribute("type", "os")

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def modify_operating_system(
        cls, operating_system_id: EntityID, *, comment: str | None = None
    ) -> Request:
        """Modifies an existing operating system.

        Args:
            operating_system_id: UUID of the operating_system to be modified.
            comment: Comment for the operating_system. Not passing a comment
                arguments clears the comment for this operating system.
        """
        if not operating_system_id:
            raise RequiredArgument(
                function=cls.modify_operating_system.__name__,
                argument="operating_system_id",
            )

        cmd = XmlCommand("modify_asset")
        cmd.set_attribute("asset_id", str(operating_system_id))
        if not comment:
            comment = ""
        cmd.add_element("comment", comment)

        return cmd



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

import httpx

from gvm.protocols.http.openvasd._metadata import (
    Metadata,
    MetadataAPI,
    MetadataError,
)


def _mock_head_response(status_code=200, headers=None):
    response = MagicMock(spec=httpx.Response)
    response.status_code = status_code
    response.headers = headers or {}
    response.raise_for_status = MagicMock()
    return response


class TestMetadataAPI(unittest.TestCase):
    def setUp(self):
        self.mock_client = MagicMock(spec=httpx.Client)

    def test_get_successful(self):
        headers = {
            "api-version": "1.0",
            "feed-version": "2025.01",
            "authentication": "API-KEY",
        }
        mock_response = _mock_head_response(200, headers)
        self.mock_client.head.return_value = mock_response

        api = MetadataAPI(self.mock_client)
        result = api.get()
        self.mock_client.head.assert_called_once_with("/")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(
            result,
            Metadata(
                api_version="1.0",
                feed_version="2025.01",
                authentication="API-KEY",
            ),
        )

    def test_get_unauthorized_with_suppress_exceptions(self):
        mock_response = MagicMock(spec=httpx.Response)
        mock_response.status_code = 401

        self.mock_client.head.side_effect = httpx.HTTPStatusError(
            "Unauthorized", request=MagicMock(), response=mock_response
        )

        api = MetadataAPI(self.mock_client, suppress_exceptions=True)
        result = api.get()
        self.assertEqual(
            result, MetadataError(error="Unauthorized", status_code=401)
        )

    def test_get_failure_raises_httpx_error(self):
        mock_response = _mock_head_response(500, None)
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.head.return_value = mock_response

        api = MetadataAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get()

        self.mock_client.head.assert_called_once_with("/")

    def test_get_scans_successful(self):
        headers = {
            "api-version": "1.0",
            "feed-version": "2025.01",
            "authentication": "API-KEY",
        }
        mock_response = _mock_head_response(200, headers)
        self.mock_client.head.return_value = mock_response

        api = MetadataAPI(self.mock_client)
        result = api.get_scans()
        self.mock_client.head.assert_called_once_with("/scans")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(
            result,
            Metadata(
                api_version="1.0",
                feed_version="2025.01",
                authentication="API-KEY",
            ),
        )

    def test_get_scans_unauthorized_with_suppress_exceptions(self):
        mock_response = MagicMock(spec=httpx.Response)
        mock_response.status_code = 401

        self.mock_client.head.side_effect = httpx.HTTPStatusError(
            "Unauthorized", request=MagicMock(), response=mock_response
        )

        api = MetadataAPI(self.mock_client, suppress_exceptions=True)
        result = api.get_scans()
        self.assertEqual(
            result, MetadataError(error="Unauthorized", status_code=401)
        )

    def test_get_scans_failure_raises_httpx_error(self):
        mock_response = _mock_head_response(500, None)
        mock_response.raise_for_status.side_effect = httpx.HTTPStatusError(
            "Server failed",
            request=MagicMock(),
            response=MagicMock(status_code=500),
        )
        self.mock_client.head.return_value = mock_response

        api = MetadataAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get_scans()

        self.mock_client.head.assert_called_once_with("/scans")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import HostsOrdering, Tasks


class TasksTestCase(unittest.TestCase):
    def test_clone_task(self):
        request = Tasks().clone_task("task_id")
        self.assertEqual(
            bytes(request),
            b"<create_task><copy>task_id</copy></create_task>",
        )

    def test_clone_task_invalid(self):
        with self.assertRaises(RequiredArgument):
            Tasks().clone_task(None)

        with self.assertRaises(RequiredArgument):
            Tasks().clone_task("")

    def test_create_container_task(self):
        request = Tasks().create_container_task("name")
        self.assertEqual(
            bytes(request),
            b'<create_task><name>name</name><target id="0"/></create_task>',
        )

    def test_create_container_task_with_comment(self):
        request = Tasks().create_container_task("name", comment="comment")
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b'<target id="0"/>'
            b"<comment>comment</comment>"
            b"</create_task>",
        )

    def test_create_task(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"</create_task>",
        )

    def test_create_task_with_comment(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            comment="comment",
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"<comment>comment</comment>"
            b"</create_task>",
        )

    def test_create_task_with_alterable(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            alterable=True,
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"<alterable>1</alterable>"
            b"</create_task>",
        )

        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            alterable=False,
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"<alterable>0</alterable>"
            b"</create_task>",
        )

    def test_create_task_with_hosts_ordering(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            hosts_ordering="random",
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</create_task>",
        )

        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            hosts_ordering=HostsOrdering.RANDOM,
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</create_task>",
        )

    def test_create_task_with_alert_ids(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            alert_ids=["alert_id1", "alert_id2"],
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b'<alert id="alert_id1"/>'
            b'<alert id="alert_id2"/>'
            b"</create_task>",
        )

    def test_create_task_with_schedule_id(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            schedule_id="schedule_id",
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b'<schedule id="schedule_id"/>'
            b"</create_task>",
        )

    def test_create_task_with_scanner_id_and_schedule_periods(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            schedule_id="schedule_id",
            schedule_periods=5,
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b'<schedule id="schedule_id"/>'
            b"<schedule_periods>5</schedule_periods>"
            b"</create_task>",
        )

    def test_create_task_with_scanner_id_and_invalid_schedule_periods(self):
        with self.assertRaises(InvalidArgument):
            Tasks().create_task(
                "name",
                target_id="target_id",
                config_id="config_id",
                scanner_id="scanner_id",
                schedule_id="schedule_id",
                schedule_periods=-1,
            )
        with self.assertRaises(InvalidArgument):
            Tasks().create_task(
                "name",
                target_id="target_id",
                config_id="config_id",
                scanner_id="scanner_id",
                schedule_id="schedule_id",
                schedule_periods="invalid",
            )

    def test_create_task_with_observers(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            observers=["observer_id1", "observer_id2"],
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"<observers>observer_id1,observer_id2</observers>"
            b"</create_task>",
        )

    def test_create_task_with_preferences(self):
        request = Tasks().create_task(
            "name",
            target_id="target_id",
            config_id="config_id",
            scanner_id="scanner_id",
            preferences={"key1": "value1", "key2": "value2"},
        )
        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>name</name>"
            b"<usage_type>scan</usage_type>"
            b'<config id="config_id"/>'
            b'<target id="target_id"/>'
            b'<scanner id="scanner_id"/>'
            b"<preferences>"
            b"<preference>"
            b"<scanner_name>key1</scanner_name>"
            b"<value>value1</value>"
            b"</preference>"
            b"<preference>"
            b"<scanner_name>key2</scanner_name>"
            b"<value>value2</value>"
            b"</preference>"
            b"</preferences>"
            b"</create_task>",
        )

    def test_create_task_invalid_hosts_ordering(self):
        with self.assertRaises(InvalidArgument):
            Tasks().create_task(
                "name",
                target_id="target_id",
                config_id="config_id",
                scanner_id="scanner_id",
                hosts_ordering="invalid",
            )

    def test_create_task_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                "",
                target_id="target_id",
                config_id="config_id",
                scanner_id="scanner_id",
            )
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                None,
                target_id="target_id",
                config_id="config_id",
                scanner_id="scanner_id",
            )

    def test_create_task_missing_target_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                "name",
                target_id="",
                config_id="config_id",
                scanner_id="scanner_id",
            )
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                "name",
                target_id=None,
                config_id="config_id",
                scanner_id="scanner_id",
            )

    def test_create_task_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                "name",
                target_id="target_id",
                config_id="",
                scanner_id="scanner_id",
            )
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                "name",
                target_id="target_id",
                config_id=None,
                scanner_id="scanner_id",
            )

    def test_create_task_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                "name",
                target_id="target_id",
                config_id="config_id",
                scanner_id="",
            )
        with self.assertRaises(RequiredArgument):
            Tasks().create_task(
                "name",
                target_id="target_id",
                config_id="config_id",
                scanner_id=None,
            )

    def test_delete_task(self):
        request = Tasks().delete_task("task_id")
        self.assertEqual(
            bytes(request),
            b'<delete_task task_id="task_id" ultimate="0"/>',
        )

    def test_delete_task_with_ultimate(self):
        request = Tasks().delete_task("task_id", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_task task_id="task_id" ultimate="1"/>',
        )

        request = Tasks().delete_task("task_id", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_task task_id="task_id" ultimate="0"/>',
        )

    def test_delete_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().delete_task("")

        with self.assertRaises(RequiredArgument):
            Tasks().delete_task(None)

    def test_get_tasks(self):
        request = Tasks().get_tasks()
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan"/>',
        )

    def test_get_tasks_with_filter_string(self):
        request = Tasks().get_tasks(filter_string="filter_string")
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" filter="filter_string"/>',
        )

    def test_get_tasks_with_filter_id(self):
        request = Tasks().get_tasks(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" filt_id="filter_id"/>',
        )

    def test_get_tasks_with_trash(self):
        request = Tasks().get_tasks(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" trash="1"/>',
        )

        request = Tasks().get_tasks(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" trash="0"/>',
        )

    def test_get_tasks_with_details(self):
        request = Tasks().get_tasks(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" details="1"/>',
        )

        request = Tasks().get_tasks(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" details="0"/>',
        )

    def test_get_tasks_with_schedules_only(self):
        request = Tasks().get_tasks(schedules_only=True)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" schedules_only="1"/>',
        )

        request = Tasks().get_tasks(schedules_only=False)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" schedules_only="0"/>',
        )

    def test_get_tasks_with_ignore_pagination(self):
        request = Tasks().get_tasks(ignore_pagination=True)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" ignore_pagination="1"/>',
        )

        request = Tasks().get_tasks(ignore_pagination=False)
        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="scan" ignore_pagination="0"/>',
        )

    def test_get_task(self):
        request = Tasks().get_task("task_id")
        self.assertEqual(
            bytes(request),
            b'<get_tasks task_id="task_id" usage_type="scan" details="1"/>',
        )

    def test_get_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().get_task("")

        with self.assertRaises(RequiredArgument):
            Tasks().get_task(None)

    def test_modify_task(self):
        request = Tasks().modify_task("task_id")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id"/>',
        )

    def test_modify_task_with_name(self):
        request = Tasks().modify_task("task_id", name="name")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id"><name>name</name></modify_task>',
        )

    def test_modify_task_with_config_id(self):
        request = Tasks().modify_task("task_id", config_id="config_id")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b'<config id="config_id"/>'
            b"</modify_task>",
        )

    def test_modify_task_with_comment(self):
        request = Tasks().modify_task("task_id", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<comment>comment</comment>"
            b"</modify_task>",
        )

    def test_modify_task_with_target_id(self):
        request = Tasks().modify_task("task_id", target_id="target_id")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b'<target id="target_id"/>'
            b"</modify_task>",
        )

    def test_modify_task_with_alterable(self):
        request = Tasks().modify_task("task_id", alterable=True)
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<alterable>1</alterable>"
            b"</modify_task>",
        )

        request = Tasks().modify_task("task_id", alterable=False)
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<alterable>0</alterable>"
            b"</modify_task>",
        )

    def test_modify_task_with_hosts_ordering(self):
        request = Tasks().modify_task("task_id", hosts_ordering="random")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</modify_task>",
        )

        request = Tasks().modify_task(
            "task_id", hosts_ordering=HostsOrdering.RANDOM
        )
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</modify_task>",
        )

    def test_modify_task_with_scanner_id(self):
        request = Tasks().modify_task("task_id", scanner_id="scanner_id")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b'<scanner id="scanner_id"/>'
            b"</modify_task>",
        )

    def test_modify_task_with_schedule_id(self):
        request = Tasks().modify_task("task_id", schedule_id="schedule_id")
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b'<schedule id="schedule_id"/>'
            b"</modify_task>",
        )

    def test_modify_task_with_schedule_periods(self):
        request = Tasks().modify_task("task_id", schedule_periods=5)
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<schedule_periods>5</schedule_periods>"
            b"</modify_task>",
        )

    def test_modify_task_with_alert_ids(self):
        request = Tasks().modify_task(
            "task_id", alert_ids=["alert_id1", "alert_id2"]
        )
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b'<alert id="alert_id1"/>'
            b'<alert id="alert_id2"/>'
            b"</modify_task>",
        )

        request = Tasks().modify_task("task_id", alert_ids=[])
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id"><alert id="0"/></modify_task>',
        )

    def test_modify_task_with_observers(self):
        request = Tasks().modify_task(
            "task_id", observers=["observer_id1", "observer_id2"]
        )
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<observers>observer_id1,observer_id2</observers>"
            b"</modify_task>",
        )
        request = Tasks().modify_task("task_id", observers=[])
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<observers></observers>"
            b"</modify_task>",
        )

    def test_modify_task_with_preferences(self):
        request = Tasks().modify_task(
            "task_id", preferences={"key1": "value1", "key2": "value2"}
        )
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id">'
            b"<preferences>"
            b"<preference>"
            b"<scanner_name>key1</scanner_name>"
            b"<value>value1</value>"
            b"</preference>"
            b"<preference>"
            b"<scanner_name>key2</scanner_name>"
            b"<value>value2</value>"
            b"</preference>"
            b"</preferences>"
            b"</modify_task>",
        )

        request = Tasks().modify_task("task_id", preferences={})
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="task_id"><preferences/></modify_task>',
        )

    def test_modify_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().modify_task("")

        with self.assertRaises(RequiredArgument):
            Tasks().modify_task(None)

    def test_modify_task_invalid_schedule_periods(self):
        with self.assertRaises(InvalidArgument):
            Tasks().modify_task("task_id", schedule_periods=-1)
        with self.assertRaises(InvalidArgument):
            Tasks().modify_task("task_id", schedule_periods="invalid")

    def test_modify_task_invalid_hosts_ordering(self):
        with self.assertRaises(InvalidArgument):
            Tasks().modify_task("task_id", hosts_ordering="invalid")

    def test_move_task(self):
        request = Tasks().move_task("task_id")
        self.assertEqual(
            bytes(request),
            b'<move_task task_id="task_id"/>',
        )

    def test_move_task_with_slave_id(self):
        request = Tasks().move_task("task_id", slave_id="slave_id")
        self.assertEqual(
            bytes(request),
            b'<move_task task_id="task_id" slave_id="slave_id"/>',
        )

    def test_move_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().move_task("")

        with self.assertRaises(RequiredArgument):
            Tasks().move_task(None)

    def test_start_task(self):
        request = Tasks().start_task("task_id")
        self.assertEqual(
            bytes(request),
            b'<start_task task_id="task_id"/>',
        )

    def test_start_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().start_task("")

        with self.assertRaises(RequiredArgument):
            Tasks().start_task(None)

    def test_resume_task(self):
        request = Tasks().resume_task("task_id")
        self.assertEqual(
            bytes(request),
            b'<resume_task task_id="task_id"/>',
        )

    def test_resume_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().resume_task("")

        with self.assertRaises(RequiredArgument):
            Tasks().resume_task(None)

    def test_stop_task(self):
        request = Tasks().stop_task("task_id")
        self.assertEqual(
            bytes(request),
            b'<stop_task task_id="task_id"/>',
        )

    def test_stop_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            Tasks().stop_task("")

        with self.assertRaises(RequiredArgument):
            Tasks().stop_task(None)



# SPDX-FileCopyrightText: 2018-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetTasksTestMixin:
    def test_get_tasks_simple(self):
        self.gmp.get_tasks()

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan"/>'
        )

    def test_get_tasks_with_filter_string(self):
        self.gmp.get_tasks(filter_string="name=foo")

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" filter="name=foo"/>'
        )

    def test_get_tasks_with_filter_id(self):
        self.gmp.get_tasks(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" filt_id="f1"/>'
        )

    def test_get_tasks_from_trash(self):
        self.gmp.get_tasks(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" trash="1"/>'
        )

    def test_get_tasks_with_details(self):
        self.gmp.get_tasks(details=True)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" details="1"/>'
        )

    def test_get_tasks_without_details(self):
        self.gmp.get_tasks(details=False)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" details="0"/>'
        )

    def test_get_tasks_with_schedules_only(self):
        self.gmp.get_tasks(schedules_only=True)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" schedules_only="1"/>'
        )

    def test_get_tasks_with_ignore_pagination(self):
        self.gmp.get_tasks(ignore_pagination=True)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" ignore_pagination="1"/>'
        )

        self.gmp.get_tasks(ignore_pagination=False)

        self.connection.send.has_been_called_with(
            b'<get_tasks usage_type="scan" ignore_pagination="0"/>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v225 import TicketStatus


class GetTicketStatusFromStringTestCase(unittest.TestCase):
    def test_invalid_status(self):
        with self.assertRaises(InvalidArgument):
            TicketStatus.from_string("foo")

    def test_none_or_empty_type(self):
        ts = TicketStatus.from_string(None)
        self.assertIsNone(ts)
        ts = TicketStatus.from_string("")
        self.assertIsNone(ts)

    def test_ticket_status_open(self):
        ts = TicketStatus.from_string("open")
        self.assertEqual(ts, TicketStatus.OPEN)

    def test_ticket_status_fixed(self):
        ts = TicketStatus.from_string("fixed")
        self.assertEqual(ts, TicketStatus.FIXED)

    def test_ticket_status_closed(self):
        ts = TicketStatus.from_string("closed")
        self.assertEqual(ts, TicketStatus.CLOSED)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreateUserTestMixin:
    def test_create_user_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_user(name=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_user(name="")

    def test_create_user(self):
        self.gmp.create_user(name="foo")

        self.connection.send.has_been_called_with(
            b"<create_user><name>foo</name></create_user>"
        )

    def test_create_user_with_password(self):
        self.gmp.create_user(name="foo", password="bar")

        self.connection.send.has_been_called_with(
            b"<create_user>"
            b"<name>foo</name>"
            b"<password>bar</password>"
            b"</create_user>"
        )

    def test_create_user_with_hosts(self):
        self.gmp.create_user(name="foo", hosts=["h1", "h2"], hosts_allow=True)

        self.connection.send.has_been_called_with(
            b"<create_user>"
            b"<name>foo</name>"
            b'<hosts allow="1">h1,h2</hosts>'
            b"</create_user>"
        )

        self.gmp.create_user(name="foo", hosts=["h1", "h2"])

        self.connection.send.has_been_called_with(
            b"<create_user>"
            b"<name>foo</name>"
            b'<hosts allow="0">h1,h2</hosts>'
            b"</create_user>"
        )

        self.gmp.create_user(name="foo", hosts=["h1", "h2"], hosts_allow=False)

        self.connection.send.has_been_called_with(
            b"<create_user>"
            b"<name>foo</name>"
            b'<hosts allow="0">h1,h2</hosts>'
            b"</create_user>"
        )

    def test_create_user_with_role_ids(self):
        self.gmp.create_user(name="foo", role_ids=["r1", "r2"])

        self.connection.send.has_been_called_with(
            b"<create_user>"
            b"<name>foo</name>"
            b'<role id="r1"/>'
            b'<role id="r2"/>'
            b"</create_user>"
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError
from gvm.protocols.gmp.requests.v224 import ReportFormatType


class GmpVerifyReportFormatTestMixin:
    def test_verify(self):
        self.gmp.verify_report_format("a1")

        self.connection.send.has_been_called_with(
            b'<verify_report_format report_format_id="a1"/>'
        )

    def test_missing_id(self):
        with self.assertRaises(GvmError):
            self.gmp.verify_report_format(None)

        with self.assertRaises(GvmError):
            self.gmp.verify_report_format("")

    def test_verify_with_type(self):
        self.gmp.verify_report_format(ReportFormatType.SVG)

        report_format_id = ReportFormatType.from_string("svg").value
        self.connection.send.has_been_called_with(
            f'<verify_report_format report_format_id="{report_format_id}"/>'.encode()
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class ResourceType(Enum):
    """Enum for resource types"""

    ALERT = "ALERT"
    AUDIT = "TASK"
    AUDIT_REPORT = "REPORT"
    CERT_BUND_ADV = "CERT_BUND_ADV"
    CONFIG = "CONFIG"
    CPE = "CPE"
    CREDENTIAL = "CREDENTIAL"
    CVE = "CVE"
    DFN_CERT_ADV = "DFN_CERT_ADV"
    FILTER = "FILTER"
    GROUP = "GROUP"
    HOST = "HOST"
    NOTE = "NOTE"
    NVT = "NVT"
    OS = "OS"
    OVERRIDE = "OVERRIDE"
    PERMISSION = "PERMISSION"
    PORT_LIST = "PORT_LIST"
    REPORT_FORMAT = "REPORT_FORMAT"
    REPORT = "REPORT"
    REPORT_CONFIG = "REPORT_CONFIG"
    RESULT = "RESULT"
    ROLE = "ROLE"
    SCANNER = "SCANNER"
    SCHEDULE = "SCHEDULE"
    TARGET = "TARGET"
    TASK = "TASK"
    TLS_CERTIFICATE = "TLS_CERTIFICATE"
    USER = "USER"


class ResourceNames:
    @classmethod
    def get_resource_names(
        cls,
        resource_type: ResourceType | str,
        *,
        filter_string: str | None = None,
    ) -> Request:
        """Request a list of resource names and IDs

        Args:
            resource_type: Type must be either ALERT, CERT_BUND_ADV,
                CONFIG, CPE, CREDENTIAL, CVE, DFN_CERT_ADV, FILTER,
                GROUP, HOST, NOTE, NVT, OS, OVERRIDE, PERMISSION,
                PORT_LIST, REPORT_FORMAT, REPORT, REPORT_CONFIG, RESULT, ROLE,
                SCANNER, SCHEDULE, TARGET, TASK, TLS_CERTIFICATE
                or USER
            filter_string: Filter term to use for the query
        """
        cmd = XmlCommand("get_resource_names")

        if not resource_type:
            raise RequiredArgument(
                function=cls.get_resource_names.__name__,
                argument="resource_type",
            )

        if not isinstance(resource_type, ResourceType):
            resource_type = ResourceType(resource_type)

        cmd.set_attribute("type", resource_type.value)
        cmd.add_filter(filter_string, None)
        return cmd

    @classmethod
    def get_resource_name(
        cls,
        resource_id: EntityID,
        resource_type: ResourceType | str,
    ) -> Request:
        """Request a single resource name

        Args:
            resource_id: ID of an existing resource
            resource_type: Type must be either ALERT, CERT_BUND_ADV,
                CONFIG, CPE, CREDENTIAL, CVE, DFN_CERT_ADV, FILTER,
                GROUP, HOST, NOTE, NVT, OS, OVERRIDE, PERMISSION,
                PORT_LIST, REPORT_FORMAT, REPORT, REPORT_CONFIG, RESULT, ROLE,
                SCANNER, SCHEDULE, TARGET, TASK, TLS_CERTIFICATE
                or USER
        """
        if not resource_type:
            raise RequiredArgument(
                function=cls.get_resource_name.__name__,
                argument="resource_type",
            )

        if not isinstance(resource_type, ResourceType):
            resource_type = ResourceType(resource_type)

        if not resource_id:
            raise RequiredArgument(
                function=cls.get_resource_name.__name__, argument="resource_id"
            )

        cmd = XmlCommand("get_resource_names")
        cmd.set_attribute("resource_id", str(resource_id))
        cmd.set_attribute("type", resource_type.value)
        return cmd



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

import httpx

from gvm.protocols.http.openvasd._health import HealthAPI


def _mock_response(status_code=200):
    mock_response = MagicMock(spec=httpx.Response)
    mock_response.status_code = status_code
    mock_response.raise_for_status = MagicMock()
    return mock_response


class TestHealthAPI(unittest.TestCase):
    def setUp(self):
        self.mock_client = MagicMock(spec=httpx.Client)

    def test_alive_returns_status_code(self):
        mock_response = _mock_response(200)
        self.mock_client.get.return_value = mock_response

        health_api = HealthAPI(self.mock_client)
        result = health_api.get_alive()
        self.mock_client.get.assert_called_once_with("/health/alive")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(result, 200)

    def test_alive_raises_httpx_error_returns_503_with_suppress_exceptions(
        self,
    ):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Not OK", request=MagicMock(), response=MagicMock(status_code=503)
        )

        health_api = HealthAPI(self.mock_client, suppress_exceptions=True)
        result = health_api.get_alive()
        self.assertEqual(result, 503)

    def test_alive_raises_httpx_error(self):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Not OK", request=MagicMock(), response=MagicMock(status_code=503)
        )
        health_api = HealthAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            health_api.get_alive()

    def test_ready_returns_status_code(self):
        mock_response = _mock_response(204)
        self.mock_client.get.return_value = mock_response

        health_api = HealthAPI(self.mock_client)
        result = health_api.get_ready()
        self.mock_client.get.assert_called_once_with("/health/ready")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(result, 204)

    def test_ready_raises_httpx_error_returns_503_with_suppress_exceptions(
        self,
    ):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Not OK", request=MagicMock(), response=MagicMock(status_code=503)
        )

        health_api = HealthAPI(self.mock_client, suppress_exceptions=True)
        result = health_api.get_ready()
        self.assertEqual(result, 503)

    def test_ready_raises_httpx_error(self):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Not OK", request=MagicMock(), response=MagicMock(status_code=503)
        )
        health_api = HealthAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            health_api.get_ready()

    def test_started_returns_status_code(self):
        mock_response = _mock_response(202)
        self.mock_client.get.return_value = mock_response

        health_api = HealthAPI(self.mock_client)
        result = health_api.get_started()
        self.mock_client.get.assert_called_once_with("/health/started")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(result, 202)

    def test_started_raises_httpx_error_returns_503_with_suppress_exceptions(
        self,
    ):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Not OK", request=MagicMock(), response=MagicMock(status_code=503)
        )

        health_api = HealthAPI(self.mock_client, suppress_exceptions=True)
        result = health_api.get_started()
        self.assertEqual(result, 503)

    def test_started_raises_httpx_error(self):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Not OK", request=MagicMock(), response=MagicMock(status_code=503)
        )

        health_api = HealthAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            health_api.get_started()



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_get_protocol_version import GmpGetProtocolVersionTestCase
from .test_get_version import GmpGetVersionTestCase

__all__ = ("GmpGetProtocolVersionTestCase", "GmpGetVersionTestCase")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from collections.abc import Sequence
from dataclasses import dataclass

from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ..v224._report_formats import ReportFormatType


@dataclass
class ReportConfigParameter:
    """
    A class to represent a parameter for a report configuration.

    Args:
        name: The name of the parameter.
        value: The value of the parameter.
        use_default (optional): A flag indicating whether to use the default value (default is False).
    """

    name: str
    value: str | None = None
    use_default: bool = False


class ReportConfigs:
    @classmethod
    def clone_report_config(cls, report_config_id: EntityID) -> Request:
        """Clone a report config from an existing one

        Args:
            report_config_id: UUID of the existing report config

        Example:

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import ReportConfigs

                request = ReportConfigs.clone_report_config("report_config_id")
        """
        if not report_config_id:
            raise RequiredArgument(
                function=cls.clone_report_config.__name__,
                argument="report_config_id",
            )

        cmd = XmlCommand("create_report_config")
        cmd.add_element("copy", str(report_config_id))
        return cmd

    @classmethod
    def delete_report_config(
        cls,
        report_config_id: EntityID,
        *,
        ultimate: bool | None = False,
    ) -> Request:
        """Deletes an existing report config

        Args:
            report_config_id: UUID of the report config to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.

        Example:

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import ReportConfigs

                request = ReportConfigs.delete_report_config(
                    "report_config_id", ultimate=True
                )
        """
        if not report_config_id:
            raise RequiredArgument(
                function=cls.delete_report_config.__name__,
                argument="report_config_id",
            )

        cmd = XmlCommand("delete_report_config")

        cmd.set_attribute("report_config_id", str(report_config_id))

        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_report_configs(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of report configs

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan report configs instead
            details: Include report config details

        Examples:

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import ReportConfigs

                request = ReportConfigs.get_report_configs()

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import ReportConfigs

                request = ReportConfigs.get_report_configs(
                    filter_string="name=foo", details=True
                )
        """
        cmd = XmlCommand("get_report_configs")

        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_report_config(
        cls,
        report_config_id: EntityID,
    ) -> Request:
        """Request a single report config

        Args:
            report_config_id: UUID of an existing report config

        Example:

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import ReportConfigs

                request = ReportConfigs.get_report_config("report_config_id")
        """
        if not report_config_id:
            raise RequiredArgument(
                function=cls.get_report_config.__name__,
                argument="report_config_id",
            )

        cmd = XmlCommand("get_report_configs")

        cmd.set_attribute("report_config_id", str(report_config_id))

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd

    @classmethod
    def create_report_config(
        cls,
        name: str,
        report_format_id: EntityID | ReportFormatType,
        *,
        comment: str | None = None,
        params: Sequence[ReportConfigParameter] | None = None,
    ) -> Request:
        """Create a report config

        Args:
            name: The name of the report config.
            report_format_id: UUID of the report format to be used or ReportFormatType.
            comment: An optional comment for the report config.
            params: A list of report config parameters.

        Example:

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import (
                    ReportConfigs,
                    ReportFormatType,
                )

                request = ReportConfigs.create_report_config(
                    "Adjusted Node Distance",
                    ReportFormatType.SVG,
                    params=[
                        ReportConfigParameter("Graph Type", use_default=True),
                        ReportConfigParameter("Node Distance", "5"),
                    ],
                )
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_report_config.__name__,
                argument="name",
            )
        if not report_format_id:
            raise RequiredArgument(
                function=cls.create_report_config.__name__,
                argument="report_format_id",
            )

        cmd = XmlCommand("create_report_config")
        cmd.add_element("name", name)
        cmd.add_element("report_format", attrs={"id": str(report_format_id)})
        if comment:
            cmd.add_element("comment", comment)

        if params:
            for param in params:
                xml_param = cmd.add_element("param")
                xml_param.add_element("name", param.name)
                value = xml_param.add_element("value")
                value.set_attribute("use_default", to_bool(param.use_default))
                if param.value is not None and not param.use_default:
                    value.set_text(param.value)

        return cmd

    @classmethod
    def modify_report_config(
        cls,
        report_config_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        params: Sequence[ReportConfigParameter] | None = None,
    ) -> Request:
        """Create a report config

        Args:
            name: The name of the report config.
            report_config_id: UUID of the report config to be modified.
            comment: An optional comment for the report config.
            params: A list of report config parameters.

        Examples:

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import ReportConfigs

                request = ReportConfigs.create_report_config(
                    report_config_id,
                    params=[
                        ReportConfigParameter("Graph Type", use_default=True),
                        ReportConfigParameter("Node Distance", "5"),
                    ],
                )

            .. code-block:: python

                from gvm.protocols.gmp.requests.v226 import ReportConfigs

                request = ReportConfigs.create_report_config(
                    report_config_id,
                    name="Modified Report Config",
                )
        """
        if not report_config_id:
            raise RequiredArgument(
                function=cls.modify_report_config.__name__,
                argument="report_config_id",
            )

        cmd = XmlCommand("modify_report_config")
        cmd.set_attribute("report_config_id", str(report_config_id))

        if name:
            cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if params:
            for param in params:
                xml_param = cmd.add_element("param")
                xml_param.add_element("name", param.name)
                value = xml_param.add_element("value")
                value.set_attribute("use_default", to_bool(param.use_default))
                if param.value is not None and not param.use_default:
                    value.set_text(param.value)

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument


class GmpGetSystemReportsTestMixin:
    def test_get_system_reports(self):
        self.gmp.get_system_reports()

        self.connection.send.has_been_called_with(b"<get_system_reports/>")

    def test_get_system_reports_with_name(self):
        self.gmp.get_system_reports(name="foo")

        self.connection.send.has_been_called_with(
            b'<get_system_reports name="foo"/>'
        )

    def test_get_system_reports_with_slave_id(self):
        self.gmp.get_system_reports(slave_id="s1")

        self.connection.send.has_been_called_with(
            b'<get_system_reports slave_id="s1"/>'
        )

    def test_get_system_reports_with_brief(self):
        self.gmp.get_system_reports(brief=True)

        self.connection.send.has_been_called_with(
            b'<get_system_reports brief="1"/>'
        )

        self.gmp.get_system_reports(brief=False)

        self.connection.send.has_been_called_with(
            b'<get_system_reports brief="0"/>'
        )

    def test_get_system_reports_with_duration(self):
        self.gmp.get_system_reports(duration=3600)

        self.connection.send.has_been_called_with(
            b'<get_system_reports duration="3600"/>'
        )

    def test_get_system_reports_with_invalid_duration(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.get_system_reports(duration="")

    def test_get_system_reports_with_start_time(self):
        self.gmp.get_system_reports(start_time="01-01-2019")

        self.connection.send.has_been_called_with(
            b'<get_system_reports start_time="01-01-2019"/>'
        )

    def test_get_system_reports_with_end_time(self):
        self.gmp.get_system_reports(end_time="01-01-2019")

        self.connection.send.has_been_called_with(
            b'<get_system_reports end_time="01-01-2019"/>'
        )



#  SPDX-FileCopyrightText: 2026 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyIntegrationConfigTestMixin:
    def test_modify_integration_config(self):
        self.gmp.modify_integration_config(integration_config_id="ic1")

        self.connection.send.has_been_called_with(
            b'<modify_integration_config uuid="ic1">'
            b"<service>"
            b"<url/>"
            b"<cacert/>"
            b"</service>"
            b"<oidc>"
            b"<oidc_provider_url/>"
            b"<client>"
            b"<id/>"
            b"<secret/>"
            b"</client>"
            b"</oidc>"
            b"</modify_integration_config>"
        )

    def test_modify_integration_config_missing_integration_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_integration_config(integration_config_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_integration_config(integration_config_id="")

    def test_modify_integration_config_with_service_url(self):
        self.gmp.modify_integration_config(
            integration_config_id="ic1",
            service_url="https://service.example.com",
        )

        self.connection.send.has_been_called_with(
            b'<modify_integration_config uuid="ic1">'
            b"<service>"
            b"<url>https://service.example.com</url>"
            b"<cacert/>"
            b"</service>"
            b"<oidc>"
            b"<oidc_provider_url/>"
            b"<client>"
            b"<id/>"
            b"<secret/>"
            b"</client>"
            b"</oidc>"
            b"</modify_integration_config>"
        )

    def test_modify_integration_config_with_service_cacert(self):
        self.gmp.modify_integration_config(
            integration_config_id="ic1",
            service_cacert="-----BEGIN CERTIFICATE-----foo-----END CERTIFICATE-----",
        )

        self.connection.send.has_been_called_with(
            b'<modify_integration_config uuid="ic1">'
            b"<service>"
            b"<url/>"
            b"<cacert>-----BEGIN CERTIFICATE-----foo-----END CERTIFICATE-----</cacert>"
            b"</service>"
            b"<oidc>"
            b"<oidc_provider_url/>"
            b"<client>"
            b"<id/>"
            b"<secret/>"
            b"</client>"
            b"</oidc>"
            b"</modify_integration_config>"
        )

    def test_modify_integration_config_with_oidc_provider_url(self):
        self.gmp.modify_integration_config(
            integration_config_id="ic1",
            oidc_provider_url="https://oidc.example.com",
        )

        self.connection.send.has_been_called_with(
            b'<modify_integration_config uuid="ic1">'
            b"<service>"
            b"<url/>"
            b"<cacert/>"
            b"</service>"
            b"<oidc>"
            b"<oidc_provider_url>https://oidc.example.com</oidc_provider_url>"
            b"<client>"
            b"<id/>"
            b"<secret/>"
            b"</client>"
            b"</oidc>"
            b"</modify_integration_config>"
        )

    def test_modify_integration_config_with_oidc_provider_client_id(self):
        self.gmp.modify_integration_config(
            integration_config_id="ic1",
            oidc_provider_client_id="client-id-1",
        )

        self.connection.send.has_been_called_with(
            b'<modify_integration_config uuid="ic1">'
            b"<service>"
            b"<url/>"
            b"<cacert/>"
            b"</service>"
            b"<oidc>"
            b"<oidc_provider_url/>"
            b"<client>"
            b"<id>client-id-1</id>"
            b"<secret/>"
            b"</client>"
            b"</oidc>"
            b"</modify_integration_config>"
        )

    def test_modify_integration_config_with_oidc_provider_client_secret(self):
        self.gmp.modify_integration_config(
            integration_config_id="ic1",
            oidc_provider_client_secret="secret-1",
        )

        self.connection.send.has_been_called_with(
            b'<modify_integration_config uuid="ic1">'
            b"<service>"
            b"<url/>"
            b"<cacert/>"
            b"</service>"
            b"<oidc>"
            b"<oidc_provider_url/>"
            b"<client>"
            b"<id/>"
            b"<secret>secret-1</secret>"
            b"</client>"
            b"</oidc>"
            b"</modify_integration_config>"
        )

    def test_modify_integration_config_with_all_fields(self):
        self.gmp.modify_integration_config(
            integration_config_id="ic1",
            service_url="https://service.example.com",
            service_cacert="cacert-data",
            oidc_provider_url="https://oidc.example.com",
            oidc_provider_client_id="client-id-1",
            oidc_provider_client_secret="secret-1",
        )

        self.connection.send.has_been_called_with(
            b'<modify_integration_config uuid="ic1">'
            b"<service>"
            b"<url>https://service.example.com</url>"
            b"<cacert>cacert-data</cacert>"
            b"</service>"
            b"<oidc>"
            b"<oidc_provider_url>https://oidc.example.com</oidc_provider_url>"
            b"<client>"
            b"<id>client-id-1</id>"
            b"<secret>secret-1</secret>"
            b"</client>"
            b"</oidc>"
            b"</modify_integration_config>"
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_get_feed import GmpGetFeedTestMixin
from .test_get_feeds import GmpGetFeedsTestMixin

__all__ = ("GmpGetFeedTestMixin", "GmpGetFeedsTestMixin")
